@@ -1,5 +1,5 @@
 /*
- *  $Id: data-browser-aux.c 22682 2019-11-18 14:01:44Z yeti-dn $
+ *  $Id: data-browser-aux.c 23511 2021-04-17 09:43:58Z yeti-dn $
  *  Copyright (C) 2006-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1054,6 +1054,27 @@ gwy_app_get_data_title_key_for_id(gint id)
                                       G_N_ELEMENTS(quarks), quarks);
 }
 
+/**
+ * gwy_app_get_data_base_key_for_id:
+ * @id: Numerical id of a channel in file (container).
+ *
+ * Calculates data field base visualisation quark identifier from its id.
+ *
+ * This is the common prefix for range and palette key.  It is not useful alone
+ * because it is only prefix.
+ *
+ * Returns: The quark key identifying base string prefix of channel with number @id.
+ *
+ * Since: 2.59
+ **/
+GQuark
+gwy_app_get_data_base_key_for_id(gint id)
+{
+    static GQuark quarks[CACHED_IDS] = { 0, };
+
+    return gwy_app_get_any_key_for_id(id, "/%d/base", G_N_ELEMENTS(quarks), quarks);
+}
+
 /**
  * gwy_app_get_data_range_type_key_for_id:
  * @id: Numerical id of a channel in file (container).
@@ -1589,7 +1610,7 @@ sync_one_data_item(GwyContainer *source, GwyContainer *dest,
         g_snprintf(key_from, sizeof(key_from), "/%d/base/palette", from_id);
         g_snprintf(key_to, sizeof(key_to), "/%d/base/palette", to_id);
         if (gwy_container_gis_string_by_name(source, key_from, &name))
-            gwy_container_set_string_by_name(dest, key_to, g_strdup(name));
+            gwy_container_set_const_string_by_name(dest, key_to, name);
         else if (delete_too)
             gwy_container_remove_by_name(dest, key_to);
         break;
@@ -1607,7 +1628,7 @@ sync_one_data_item(GwyContainer *source, GwyContainer *dest,
         g_snprintf(key_from, sizeof(key_from), "/%d/data/title", from_id);
         g_snprintf(key_to, sizeof(key_to), "/%d/data/title", to_id);
         if (gwy_container_gis_string_by_name(source, key_from, &name))
-            gwy_container_set_string_by_name(dest, key_to, g_strdup(name));
+            gwy_container_set_const_string_by_name(dest, key_to, name);
         else if (delete_too)
             gwy_container_remove_by_name(dest, key_to);
         break;
@@ -1795,7 +1816,7 @@ sync_one_volume_item(GwyContainer *source, GwyContainer *dest,
         g_snprintf(key_to, sizeof(key_to),
                    BRICK_PREFIX "/%d/preview/palette", to_id);
         if (gwy_container_gis_string_by_name(source, key_from, &name))
-            gwy_container_set_string_by_name(dest, key_to, g_strdup(name));
+            gwy_container_set_const_string_by_name(dest, key_to, name);
         else if (delete_too)
             gwy_container_remove_by_name(dest, key_to);
         break;
@@ -1806,7 +1827,7 @@ sync_one_volume_item(GwyContainer *source, GwyContainer *dest,
         g_snprintf(key_to, sizeof(key_to),
                    BRICK_PREFIX "/%d/title", to_id);
         if (gwy_container_gis_string_by_name(source, key_from, &name))
-            gwy_container_set_string_by_name(dest, key_to, g_strdup(name));
+            gwy_container_set_const_string_by_name(dest, key_to, name);
         else if (delete_too)
             gwy_container_remove_by_name(dest, key_to);
         break;
@@ -2042,7 +2063,7 @@ gwy_app_sync_brick_items(GwyContainer *source,
             g_snprintf(key_to, sizeof(key_to),
                        BRICK_PREFIX "/%d/preview/palette", to_id);
             if (gwy_container_gis_string_by_name(source, key_from, &name))
-                gwy_container_set_string_by_name(dest, key_to, g_strdup(name));
+                gwy_container_set_const_string_by_name(dest, key_to, name);
             else if (delete_too)
                 gwy_container_remove_by_name(dest, key_to);
             break;
@@ -2053,7 +2074,7 @@ gwy_app_sync_brick_items(GwyContainer *source,
             g_snprintf(key_to, sizeof(key_to),
                        BRICK_PREFIX "/%d/title", to_id);
             if (gwy_container_gis_string_by_name(source, key_from, &name))
-                gwy_container_set_string_by_name(dest, key_to, g_strdup(name));
+                gwy_container_set_const_string_by_name(dest, key_to, name);
             else if (delete_too)
                 gwy_container_remove_by_name(dest, key_to);
             break;
@@ -2187,7 +2208,7 @@ gwy_app_sync_surface_items(GwyContainer *source,
             g_snprintf(key_to, sizeof(key_to),
                        SURFACE_PREFIX "/%d/preview/palette", to_id);
             if (gwy_container_gis_string_by_name(source, key_from, &name))
-                gwy_container_set_string_by_name(dest, key_to, g_strdup(name));
+                gwy_container_set_const_string_by_name(dest, key_to, name);
             else if (delete_too)
                 gwy_container_remove_by_name(dest, key_to);
             break;
@@ -2198,7 +2219,7 @@ gwy_app_sync_surface_items(GwyContainer *source,
             g_snprintf(key_to, sizeof(key_to),
                        SURFACE_PREFIX "/%d/title", to_id);
             if (gwy_container_gis_string_by_name(source, key_from, &name))
-                gwy_container_set_string_by_name(dest, key_to, g_strdup(name));
+                gwy_container_set_const_string_by_name(dest, key_to, name);
             else if (delete_too)
                 gwy_container_remove_by_name(dest, key_to);
             break;
