@@ -1,5 +1,5 @@
 /*
- *  $Id: datachooser.c 22218 2019-07-11 08:01:47Z yeti-dn $
+ *  $Id: datachooser.c 23637 2021-05-05 07:32:51Z yeti-dn $
  *  Copyright (C) 2006-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -541,8 +541,18 @@ gwy_data_chooser_get_filter(GwyDataChooser *chooser)
 void
 gwy_data_chooser_refilter(GwyDataChooser *chooser)
 {
+    GtkTreeModel *model;
+    GwyContainer *data;
+    gint id;
+
     g_return_if_fail(GWY_IS_DATA_CHOOSER(chooser));
+    data = gwy_data_chooser_get_active(chooser, &id);
+    model = g_object_ref(chooser->filter);
+    gtk_combo_box_set_model(GTK_COMBO_BOX(chooser), NULL);
     gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(chooser->filter));
+    gtk_combo_box_set_model(GTK_COMBO_BOX(chooser), model);
+    g_object_unref(model);
+    gwy_data_chooser_set_active(chooser, data, id);
     gwy_data_chooser_choose_whatever(chooser);
 }
 
@@ -576,24 +586,20 @@ void
 gwy_data_chooser_set_none(GwyDataChooser *chooser,
                           const gchar *none)
 {
-    gchar *old_none;
     GtkTreeIter iter;
     Proxy *proxy;
 
     g_return_if_fail(GWY_IS_DATA_CHOOSER(chooser));
-    old_none = chooser->none_label;
-    chooser->none_label = g_strdup(none);
-    g_free(old_none);
+    gwy_assign_string(&chooser->none_label, none);
 
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(chooser->store), &iter);
     gtk_tree_model_get(GTK_TREE_MODEL(chooser->store), &iter,
                        MODEL_COLUMN_PROXY, &proxy,
                        -1);
-    g_free(proxy->name);
-    if (chooser->none_label && *chooser->none_label)
-        proxy->name = g_strdup(chooser->none_label);
-    else
-        proxy->name = g_strdup(gwy_sgettext("channel|None"));
+    gwy_assign_string(&proxy->name,
+                      chooser->none_label && *chooser->none_label
+                      ? chooser->none_label
+                      : gwy_sgettext("channel|None"));
     gwy_list_store_row_changed(chooser->store, &iter, NULL, 0);
 
     gwy_data_chooser_choose_whatever(chooser);
