@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyappfilechooser.c 22534 2019-10-02 08:35:40Z yeti-dn $
+ *  $Id: gwyappfilechooser.c 23366 2021-03-25 15:37:11Z yeti-dn $
  *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -812,7 +812,7 @@ gwy_app_file_chooser_add_preview(GwyAppFileChooser *chooser)
     gtk_icon_view_set_selection_mode(preview, GTK_SELECTION_NONE);
     /* In Gtk+ 2.14 and older, things work.  2.16 adds some padding that
      * breaks everything.  And this padding together with the usual margin
-     * meaks too much white space so we have to get rid of the margin in 2.16+.
+     * means too much white space so we have to get rid of the margin in 2.16+.
      */
     if (gtk_major_version == 2 && gtk_minor_version <= 14) {
         gtk_icon_view_set_item_width(preview, TMS_NORMAL_THUMB_SIZE);
@@ -1231,6 +1231,28 @@ describe_volume(GwyContainer *container, gint id, GString *str)
     gwy_si_unit_value_format_free(vf);
 }
 
+static void
+ensure_brick_preview(GwyContainer *container, gint id)
+{
+    GwyBrick *brick;
+    GwyDataField *preview;
+    GQuark pquark, bquark;
+
+    pquark = gwy_app_get_brick_preview_key_for_id(id);
+    if (gwy_container_gis_object(container, pquark, (GObject**)&preview)
+        && GWY_IS_DATA_FIELD(preview))
+        return;
+
+    bquark = gwy_app_get_brick_key_for_id(id);
+    brick = gwy_container_get_object(container, bquark);
+    if (!GWY_IS_BRICK(brick))
+        return;
+
+    preview = _gwy_app_create_brick_preview_field(brick);
+    gwy_container_set_object(container, pquark, preview);
+    g_object_unref(preview);
+}
+
 static void
 add_volume_thumbnails(GwyAppFileChooser *chooser,
                       GwyContainer *data, gint *ids,
@@ -1242,6 +1264,7 @@ add_volume_thumbnails(GwyAppFileChooser *chooser,
 
     for (i = 0; ids[i] != -1; i++) {
         id = ids[i];
+        ensure_brick_preview(data, id);
         pixbuf = gwy_app_get_volume_thumbnail(data, id,
                                               TMS_NORMAL_THUMB_SIZE,
                                               TMS_NORMAL_THUMB_SIZE);
