@@ -1,22 +1,18 @@
 /*
- *  $Id: gwymoduleutils.c 21435 2018-09-11 12:39:53Z yeti-dn $
- *  Copyright (C) 2007-2018 David Necas (Yeti), Petr Klapetek.
+ *  $Id: gwymoduleutils.c 23755 2021-05-21 11:16:58Z yeti-dn $
+ *  Copyright (C) 2007-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -28,9 +24,17 @@
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <libgwyddion/gwymacros.h>
-#include <app/file.h>
-#include <app/data-browser.h>
-#include <app/gwymoduleutils.h>
+#include <libgwydgets/gwylayer-basic.h>
+#include <libgwydgets/gwylayer-mask.h>
+#include <libgwydgets/gwydataview.h>
+#include <libgwydgets/gwydgetutils.h>
+#include "file.h"
+#include "data-browser.h"
+#include "settings.h"
+#include "wait.h"
+#include "dialog.h"
+#include "param-internal.h"
+#include "gwymoduleutils.h"
 
 typedef struct {
     const gchar *data;
@@ -52,20 +56,16 @@ gwy_save_auxiliary_data_create(gpointer user_data,
  * gwy_save_auxiliary_data:
  * @title: File chooser dialog title.
  * @parent: Parent window for the file chooser dialog (may be %NULL).
- * @data_len: The length of @data in bytes.  Pass -1 if @data is text, it must
- *            be nul-terminated then and it will be saved in text mode (this
- *            matters if the operating system distinguishes between text and
- *            binary).  A non-negative value causes the data to be saved as
- *            binary.
+ * @data_len: The length of @data in bytes.  Pass -1 if @data is text, it must be nul-terminated then and it will be
+ *            saved in text mode (this matters if the operating system distinguishes between text and binary).
+ *            A non-negative value causes the data to be saved as binary.
  * @data: The data to save.
  *
  * Saves a report or other auxiliary data to a user specified file.
  *
- * This is actually a simple gwy_save_auxiliary_with_callback() wrapper, see
- * its description for details.
+ * This is actually a simple gwy_save_auxiliary_with_callback() wrapper, see its description for details.
  *
- * Returns: %TRUE if the data was save, %FALSE if it was not saved for any
- *          reason.
+ * Returns: %TRUE if the data was save, %FALSE if it was not saved for any reason.
  *
  * Since: 2.3
  **/
@@ -81,26 +81,21 @@ gwy_save_auxiliary_data(const gchar *title,
     savedata.data = data;
     savedata.len = data_len;
 
-    return gwy_save_auxiliary_with_callback(title, parent,
-                                            &gwy_save_auxiliary_data_create,
-                                            NULL,
-                                            &savedata);
+    return gwy_save_auxiliary_with_callback(title, parent, &gwy_save_auxiliary_data_create, NULL, &savedata);
 }
 
 /**
  * gwy_save_auxiliary_with_callback:
  * @title: File chooser dialog title.
  * @parent: Parent window for the file chooser dialog (may be %NULL).
- * @create: Function to create the data (it will not be called if the user
- *          cancels the saving).
- * @destroy: Function to destroy the data (if will be called iff @create will
- *           be called), it may be %NULL.
+ * @create: Function to create the data (it will not be called if the user cancels the saving).
+ * @destroy: Function to destroy the data (if will be called iff @create will be called), it may be %NULL.
  * @user_data: User data passed to @create and @destroy.
  *
  * Saves a report or other auxiliary data to a user specified file.
  *
- * Returns: %TRUE if the data was save, %FALSE if it was not saved for any
- *          reason (I/O error, cancellation, overwrite cancellation, etc.).
+ * Returns: %TRUE if the data was save, %FALSE if it was not saved for any reason (I/O error, cancellation, overwrite
+ *          cancellation, etc.).
  *
  * Since: 2.3
  **/
@@ -128,8 +123,7 @@ gwy_save_auxiliary_with_callback(const gchar *title,
                                           GTK_STOCK_SAVE, GTK_RESPONSE_OK,
                                           NULL);
     gtk_dialog_set_default_response(GTK_DIALOG(chooser), GTK_RESPONSE_OK);
-    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(chooser),
-                                        gwy_app_get_current_directory());
+    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(chooser), gwy_app_get_current_directory());
     response = gtk_dialog_run(GTK_DIALOG(chooser));
     filename_sys = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
 
@@ -187,15 +181,12 @@ gwy_save_auxiliary_with_callback(const gchar *title,
     }
 
     filename_utf8 = g_filename_to_utf8(filename_sys, -1, NULL, NULL, NULL);
-    dialog = gtk_message_dialog_new(parent, 0, GTK_MESSAGE_ERROR,
-                                    GTK_BUTTONS_OK,
-                                    _("Saving of `%s' failed"),
-                                    filename_utf8);
+    dialog = gtk_message_dialog_new(parent, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+                                    _("Saving of `%s' failed"), filename_utf8);
     g_free(filename_sys);
     g_free(filename_utf8);
     gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
-                                             _("Cannot write to file: %s."),
-                                             g_strerror(myerrno));
+                                             _("Cannot write to file: %s."), g_strerror(myerrno));
     gtk_widget_show_all(dialog);
     gtk_dialog_run(GTK_DIALOG(dialog));
     gtk_widget_destroy(dialog);
@@ -206,21 +197,18 @@ gwy_save_auxiliary_with_callback(const gchar *title,
 /**
  * gwy_module_data_load:
  * @modname: Module name (determines the subdirectory).
- * @filename: Name of the file to load.  In GLib encoding, but it really should
- *            be just ASCII.
- * @contents: Location to store the allocated file contents.  Use g_free() to
- *            free it.  It is set to %NULL when the file cannot be loaded.
+ * @filename: Name of the file to load.  In GLib encoding, but it really should be just ASCII.
+ * @contents: Location to store the allocated file contents.  Use g_free() to free it.  It is set to %NULL when the
+ *            file cannot be loaded.
  * @length: Location to the length of the contents in bytes, or %NULL.
  * @error: Location for error, or %NULL.
  *
  * Load module data file from the user directory.
  *
- * The function wraps g_file_get_contents(), forming the full file name
- * automatically.
+ * The function wraps g_file_get_contents(), forming the full file name automatically.
  *
- * The error can be from %G_FILE_ERROR domain.  Usually, however, you only need
- * the return value and consider the file simply not existing yet when the
- * function fails.
+ * The error can be from %G_FILE_ERROR domain.  Usually, however, you only need the return value and consider the file
+ * simply not existing yet when the function fails.
  *
  * Returns: %TRUE if the file was loaded.
  *
@@ -261,9 +249,7 @@ ensure_module_directory(const gchar *userdir,
         return TRUE;
     }
 
-    g_set_error_literal(error, G_FILE_ERROR,
-                        g_file_error_from_errno(errno),
-                        g_strerror(errno));
+    g_set_error_literal(error, G_FILE_ERROR, g_file_error_from_errno(errno), g_strerror(errno));
     g_free(path);
     return FALSE;
 }
@@ -271,16 +257,15 @@ ensure_module_directory(const gchar *userdir,
 /**
  * gwy_module_data_save:
  * @modname: Module name (determines the subdirectory).
- * @filename: Name of the file to save.  In GLib encoding, but it really should
- *            be just ASCII.
+ * @filename: Name of the file to save.  In GLib encoding, but it really should be just ASCII.
  * @contents: File contents to write.
  * @length: Length of @contents, or -1 if it is a NUL-terminated string.
  * @error: Location for error, or %NULL.
  *
  * Saves module data file to the user directory.
  *
- * The function wraps g_file_set_contents(), forming the full file name
- * automatically and handling subdirectory creation.
+ * The function wraps g_file_set_contents(), forming the full file name automatically and handling subdirectory
+ * creation.
  *
  * The error can be from %G_FILE_ERROR domain.
  *
@@ -313,20 +298,17 @@ gwy_module_data_save(const gchar *modname,
 /**
  * gwy_module_data_fopen:
  * @modname: Module name (determines the subdirectory).
- * @filename: Name of the file to save.  In GLib encoding, but it really should
- *            be just ASCII.
+ * @filename: Name of the file to save.  In GLib encoding, but it really should be just ASCII.
  * @mode: File open mode, as in fopen().
  * @error: Location for error, or %NULL.
  *
  * Opens a module data file in the user directory.
  *
- * The function wraps gwy_fopen(), forming the full file name automatically and
- * handling subdirectory creation.
+ * The function wraps gwy_fopen(), forming the full file name automatically and handling subdirectory creation.
  *
  * The error can be from %G_FILE_ERROR domain.
  *
- * Returns: A file handle if the file was opened as requested, %NULL on
- *          failure.
+ * Returns: A file handle if the file was opened as requested, %NULL on failure.
  *
  * Since: 2.51
  **/
@@ -345,11 +327,8 @@ gwy_module_data_fopen(const gchar *modname,
         return FALSE;
 
     path = g_build_filename(userdir, modname, filename, NULL);
-    if (!(fh = gwy_fopen(path, mode))) {
-        g_set_error_literal(error, G_FILE_ERROR,
-                            g_file_error_from_errno(errno),
-                            g_strerror(errno));
-    }
+    if (!(fh = gwy_fopen(path, mode)))
+        g_set_error_literal(error, G_FILE_ERROR, g_file_error_from_errno(errno), g_strerror(errno));
     g_free(path);
 
     return fh;
@@ -362,9 +341,8 @@ gwy_module_data_fopen(const gchar *modname,
  *
  * Sets up data view zoom to not exceed specified size.
  *
- * Before calling this function, data keys have be set, data fields and layers
- * have to be present and physically square mode set in the container.
- * Sizing of both pixel-wise square and physically square displays is performed
+ * Before calling this function, data keys have be set, data fields and layers have to be present and physically
+ * square mode set in the container. Sizing of both pixel-wise square and physically square displays is performed
  * correctly.
  *
  * Since: 2.7
@@ -430,10 +408,9 @@ clear_data_id(GwyAppDataId *id)
  *
  * Checks if numerical channel identifiers correspond to an existing channel.
  *
- * If either the data contained referenced in @id or the channel does not exist
- * the structure is cleared to %GWY_APP_DATA_ID_NONE and the function returns
- * %FALSE.  If it represents an existing channel it is kept intact and the
- * function returns %TRUE.
+ * If either the data contained referenced in @id or the channel does not exist the structure is cleared to
+ * %GWY_APP_DATA_ID_NONE and the function returns %FALSE.  If it represents an existing channel it is kept intact and
+ * the function returns %TRUE.
  *
  * Returns: Whether @id refers to an existing channel now.
  *
@@ -467,9 +444,8 @@ gwy_app_data_id_verify_channel(GwyAppDataId *id)
  *
  * Checks if numerical graph identifiers correspond to an existing graph.
  *
- * If either the data contained referenced in @id or the graph model does not
- * exist the structure is cleared to %GWY_APP_DATA_ID_NONE and the function
- * returns %FALSE.  If it represents an existing graph it is kept intact and
+ * If either the data contained referenced in @id or the graph model does not exist the structure is cleared to
+ * %GWY_APP_DATA_ID_NONE and the function returns %FALSE.  If it represents an existing graph it is kept intact and
  * the function returns %TRUE.
  *
  * Returns: Whether @id refers to an existing graph now.
@@ -500,16 +476,13 @@ gwy_app_data_id_verify_graph(GwyAppDataId *id)
 
 /**
  * gwy_app_data_id_verify_volume:
- * @id: Numerical identifiers of volume data in data managed by the data
- *      browser.
+ * @id: Numerical identifiers of volume data in data managed by the data browser.
  *
- * Checks if numerical volume data identifiers correspond to existing volume
- * data.
+ * Checks if numerical volume data identifiers correspond to existing volume data.
  *
- * If either the data contained referenced in @id or the volume data does not
- * exist the structure is cleared to %GWY_APP_DATA_ID_NONE and the function
- * returns %FALSE.  If it represents existing volume data it is kept intact
- * and the function returns %TRUE.
+ * If either the data contained referenced in @id or the volume data does not exist the structure is cleared to
+ * %GWY_APP_DATA_ID_NONE and the function returns %FALSE.  If it represents existing volume data it is kept intact and
+ * the function returns %TRUE.
  *
  * Returns: Whether @id refers to existing volume data now.
  *
@@ -539,15 +512,13 @@ gwy_app_data_id_verify_volume(GwyAppDataId *id)
 
 /**
  * gwy_app_data_id_verify_xyz:
- * @id: Numerical identifiers of XYZ data in data managed by the data
- *      browser.
+ * @id: Numerical identifiers of XYZ data in data managed by the data browser.
  *
  * Checks if numerical XYZ data identifiers correspond to existing XYZ data.
  *
- * If either the data contained referenced in @id or the XYZ data does not
- * exist the structure is cleared to %GWY_APP_DATA_ID_NONE and the function
- * returns %FALSE.  If it represents existing XYZ data it is kept intact
- * and the function returns %TRUE.
+ * If either the data contained referenced in @id or the XYZ data does not exist the structure is cleared to
+ * %GWY_APP_DATA_ID_NONE and the function returns %FALSE.  If it represents existing XYZ data it is kept intact and
+ * the function returns %TRUE.
  *
  * Returns: Whether @id refers to existing XYZ data now.
  *
@@ -581,10 +552,9 @@ gwy_app_data_id_verify_xyz(GwyAppDataId *id)
  *
  * Checks if numerical spectra identifiers correspond to existing spectra.
  *
- * If either the data contained referenced in @id or the spectra does not
- * exist the structure is cleared to %GWY_APP_DATA_ID_NONE and the function
- * returns %FALSE.  If it represents existing spectra it is kept intact and
- * the function return %TRUE.
+ * If either the data contained referenced in @id or the spectra does not exist the structure is cleared to
+ * %GWY_APP_DATA_ID_NONE and the function returns %FALSE.  If it represents existing spectra it is kept intact and the
+ * function return %TRUE.
  *
  * Returns: Whether @id refers to existing spectra now.
  *
@@ -619,24 +589,20 @@ gwy_app_data_id_verify_spectra(GwyAppDataId *id)
  * @target_graph: Graph where curves would be added.
  * @colorstep: Curve block size as in gwy_graph_model_append_curves().
  *
- * Puts the curves of a graph to another graph if possible, or adds the graph
- * as new.
+ * Puts the curves of a graph to another graph if possible, or adds the graph as new.
  *
- * If the units of @gmodel are compatible with the units of the graph
- * identified by @target_graph the curves are copied to the target graph with
- * gwy_graph_model_append_curves().
+ * If the units of @gmodel are compatible with the units of the graph identified by @target_graph the curves are
+ * copied to the target graph with gwy_graph_model_append_curves().
  *
- * In all other cases, including when @target_graph does not refer to any
- * existing graph, the graph model is added to @data as a new graph.
+ * In all other cases, including when @target_graph does not refer to any existing graph, the graph model is added to
+ * @data as a new graph.
  *
  * Either way, the caller usually need to release its own reference afterwards.
  *
- * This function is useful particularly in modules that create graphs and can
- * be run non-interactively.
+ * This function is useful particularly in modules that create graphs and can be run non-interactively.
  *
- * Returns: The numerical identifier of the newly-created graph of one was
- *          created.  Value -1 is returned if curves were added to
- *          @target_graph.
+ * Returns: The numerical identifier of the newly-created graph of one was created.  Value -1 is returned if curves
+ *          were added to @target_graph.
  *
  * Since: 2.41
  **/
@@ -726,8 +692,7 @@ gwy_preview_surface_to_datafield(GwySurface *surface,
     }
 
     if (do_fill) {
-        h = fmax((xmax - xmin)/(max_xres - 1.0),
-                 (ymax - ymin)/(max_yres - 1.0));
+        h = fmax((xmax - xmin)/(max_xres - 1.0), (ymax - ymin)/(max_yres - 1.0));
         xmin -= 0.5*h;
         ymin -= 0.5*h;
         xmax += 0.5*h;
@@ -786,20 +751,318 @@ gwy_preview_surface_to_datafield(GwySurface *surface,
     if (want_densitymap)
         densitymap = gwy_data_field_new_alike(dfield, FALSE);
 
-    gwy_data_field_average_xyz(dfield, densitymap,
-                               gwy_surface_get_data_const(surface), n);
-    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield),
-                       gwy_surface_get_si_unit_xy(surface));
+    gwy_data_field_average_xyz(dfield, densitymap, gwy_surface_get_data_const(surface), n);
+    gwy_si_unit_assign(gwy_data_field_get_si_unit_xy(dfield), gwy_surface_get_si_unit_xy(surface));
 
     if (want_densitymap) {
         gwy_data_field_copy(densitymap, dfield, FALSE);
         GWY_OBJECT_UNREF(densitymap);
         gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(dfield), NULL);
     }
-    else {
-        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield),
-                           gwy_surface_get_si_unit_z(surface));
+    else
+        gwy_si_unit_assign(gwy_data_field_get_si_unit_z(dfield), gwy_surface_get_si_unit_z(surface));
+}
+
+/**
+ * gwy_app_wait_preview_data_field:
+ * @dfield: A data field used for preview.
+ * @data: Data container with source image (or %NULL).
+ * @id: Source image id (or -1).
+ *
+ * Helper function for progressive animated preview.
+ *
+ * This is a wrapper for gwy_app_wait_set_preview_widget() which creates a simple image preview displaying @dfield.
+ * Call gwy_data_field_data_changed() on it to redraw the preview.
+ *
+ * No preview is created if progress bars are disabled, see gwy_app_wait_set_enabled().
+ *
+ * Returns: The created #GwyDataView with the preview (possibly %NULL if not preview was actually created).  Usually
+ *          you can ignore it.
+ *
+ * Since: 2.59
+ **/
+GtkWidget*
+gwy_app_wait_preview_data_field(GwyDataField *dfield,
+                                GwyContainer *data,
+                                gint id)
+{
+    GwyContainer *mydata;
+    GtkWidget *preview;
+
+    if (!gwy_app_wait_get_enabled())
+        return NULL;
+
+    mydata = gwy_container_new();
+    gwy_container_set_object(mydata, gwy_app_get_data_key_for_id(0), dfield);
+    preview = gwy_create_preview(mydata, 0, 360, FALSE);
+    if (data && id >= 0) {
+        gwy_app_sync_data_items(data, mydata, id, 0, FALSE,
+                                GWY_DATA_ITEM_GRADIENT,
+                                0);
     }
+    gwy_app_wait_set_preview_widget(preview);
+
+    return preview;
+}
+
+/**
+ * gwy_require_image_same_units:
+ * @field: The data field to check.
+ * @data: Data container with the data field.
+ * @id: The id of @field in @data.
+ * @name: Error message prefix, usually module function name.
+ *
+ * Checks if data field has equal lateral and value units, possibly showing error message.
+ *
+ * If the data field does not have the same lateral and value units a simple error message dialog is shown for @data
+ * and @id.  This is only done if other GUI functions (like progress bars for waiting) are enabled.
+ *
+ * Returns: %TRUE if the units are the same, %FALSE otherwise.
+ *
+ * Since: 2.59
+ **/
+gboolean
+gwy_require_image_same_units(GwyDataField *field, GwyContainer *data, gint id,
+                             const gchar *name)
+{
+    GtkWidget *dialog;
+
+    if (gwy_si_unit_equal(gwy_data_field_get_si_unit_xy(field), gwy_data_field_get_si_unit_z(field)))
+        return TRUE;
+
+    if (!gwy_app_data_browser_get_gui_enabled() && !gwy_app_wait_get_enabled())
+        return FALSE;
+
+    dialog = gtk_message_dialog_new(gwy_app_find_window_for_channel(data, id),
+                                    GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+                                    GTK_MESSAGE_ERROR,
+                                    GTK_BUTTONS_OK,
+                                    _("%s: Lateral dimensions and value must be the same physical quantity."), name);
+    if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_NONE)
+        gtk_widget_destroy(dialog);
+
+    return FALSE;
+}
+
+/**
+ * gwy_require_square_image:
+ * @field: The data field to check.
+ * @data: Data container with the data field.
+ * @id: The id of @field in @data.
+ * @name: Error message prefix, usually module function name.
+ *
+ * Checks if data field has the same horizontal and vertical pixel dimensions, possibly showing error message.
+ *
+ * If the data field is not square a simple error message dialog is shown for @data and @id.  This is only done if
+ * other GUI functions (like progress bars for waiting) are enabled.
+ *
+ * Returns: %TRUE if the data field is square, %FALSE otherwise.
+ *
+ * Since: 2.59
+ **/
+gboolean
+gwy_require_square_image(GwyDataField *field, GwyContainer *data, gint id,
+                         const gchar *name)
+{
+    GtkWidget *dialog;
+
+    if (gwy_data_field_get_xres(field) == gwy_data_field_get_yres(field))
+        return TRUE;
+
+    if (!gwy_app_data_browser_get_gui_enabled() && !gwy_app_wait_get_enabled())
+        return FALSE;
+
+    dialog = gtk_message_dialog_new(gwy_app_find_window_for_channel(data, id),
+                                    GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+                                    GTK_MESSAGE_ERROR,
+                                    GTK_BUTTONS_OK,
+                                    _("%s: Data must be square."), name);
+    if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_NONE)
+        gtk_widget_destroy(dialog);
+
+    return FALSE;
+}
+
+static GwyPixmapLayer*
+create_basic_layer(GwyDataView *dataview, gint id)
+{
+    GwyPixmapLayer *layer = gwy_layer_basic_new();
+    GQuark quark = gwy_app_get_data_key_for_id(id);
+    const gchar *key = g_quark_to_string(quark);
+    gchar buf[24];
+
+    gwy_pixmap_layer_set_data_key(layer, key);
+    key = g_quark_to_string(gwy_app_get_data_palette_key_for_id(id));
+    gwy_layer_basic_set_gradient_key(GWY_LAYER_BASIC(layer), key);
+    key = g_quark_to_string(gwy_app_get_data_range_type_key_for_id(id));
+    gwy_layer_basic_set_range_type_key(GWY_LAYER_BASIC(layer), key);
+    g_snprintf(buf, sizeof(buf), "/%d/base", id);
+    gwy_layer_basic_set_min_max_key(GWY_LAYER_BASIC(layer), buf);
+    gwy_data_view_set_base_layer(dataview, layer);
+
+    return layer;
+}
+
+static GwyPixmapLayer*
+create_mask_layer(GwyDataView *dataview, gint id)
+{
+    GwyPixmapLayer *layer = gwy_layer_mask_new();
+    GQuark quark = gwy_app_get_mask_key_for_id(id);
+    const gchar *key = g_quark_to_string(quark);
+    GwyContainer *data;
+    GwyRGBA rgba;
+
+    gwy_pixmap_layer_set_data_key(layer, key);
+    gwy_layer_mask_set_color_key(GWY_LAYER_MASK(layer), key);
+    gwy_data_view_set_alpha_layer(dataview, layer);
+
+    data = gwy_data_view_get_data(dataview);
+    if (!gwy_rgba_get_from_container(&rgba, data, key)) {
+        gwy_rgba_get_from_container(&rgba, gwy_app_settings_get(), "/mask");
+        gwy_rgba_store_to_container(&rgba, data, key);
+    }
+
+    return layer;
+}
+
+/**
+ * gwy_create_preview:
+ * @data: Data container with images to preview.
+ * @id: Data id of preview image in @data.
+ * @size: Preview size.  Prefrably one of the standard sizes.
+ * @have_mask: %TRUE if the preview will have mask and should be set up so.
+ *
+ * Creates and sets up a preview widget for use in modules.
+ *
+ * Returns: A new data view widget with the preview.
+ *
+ * Since: 2.59
+ **/
+GtkWidget*
+gwy_create_preview(GwyContainer *data,
+                   gint id,
+                   gint size,
+                   gboolean have_mask)
+{
+    GQuark quark = gwy_app_get_data_key_for_id(id);
+    const gchar *key = g_quark_to_string(quark);
+    GwyDataView *dataview;
+    GtkWidget *widget;
+
+    widget = gwy_data_view_new(data);
+    dataview = GWY_DATA_VIEW(widget);
+    gwy_data_view_set_data_prefix(dataview, key);
+    create_basic_layer(dataview, id);
+    if (have_mask)
+        create_mask_layer(dataview, id);
+    gwy_set_data_preview_size(dataview, size);
+    g_object_set_data(G_OBJECT(widget), "gwy-preview-size", GINT_TO_POINTER(size));
+
+    return widget;
+}
+
+/**
+ * gwy_create_dialog_preview_hbox:
+ * @dialog: A data processing dialog.
+ * @dataview: Preview data view, usually created by gwy_create_preview().
+ * @pack_end: %TRUE to pack the data view to the end; %FALSE to pack it to the start.
+ *
+ * Creates the typical data processing dialog hbox with a preview.
+ *
+ * The box is added to the dialog.
+ *
+ * Returns: A new #GtkBox widget.
+ *
+ * Since: 2.59
+ **/
+GtkWidget*
+gwy_create_dialog_preview_hbox(GtkDialog *dialog,
+                               GwyDataView *dataview,
+                               gboolean pack_end)
+{
+    GtkWidget *hbox, *alignment, *vbox;
+    gint size;
+
+    hbox = gwy_hbox_new(8);
+    gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
+
+    g_return_val_if_fail(GTK_IS_DIALOG(dialog), hbox);
+    g_return_val_if_fail(GWY_IS_DATA_VIEW(dataview), hbox);
+
+    if (GWY_IS_DIALOG(dialog))
+        gwy_dialog_add_content(GWY_DIALOG(dialog), hbox, FALSE, FALSE, 0);
+    else {
+        vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+        gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+    }
+
+    alignment = gtk_alignment_new(0.0, 0.0, 0.0, 0.0);
+    if ((size = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(dataview), "gwy-preview-size"))))
+        gtk_widget_set_size_request(alignment, size, -1);
+    gtk_container_add(GTK_CONTAINER(alignment), GTK_WIDGET(dataview));
+    if (pack_end)
+        gtk_box_pack_end(GTK_BOX(hbox), alignment, FALSE, FALSE, 0);
+    else
+        gtk_box_pack_start(GTK_BOX(hbox), alignment, FALSE, FALSE, 0);
+
+    return hbox;
+}
+
+static void
+active_page_changed(GtkNotebook *notebook, G_GNUC_UNUSED gpointer some_gtk_shit, gint pageno, GwyParams *params)
+{
+    gint id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(notebook), "gwy-active-page-param-id"));
+    gwy_params_set_int(params, id, pageno);
+}
+
+static void
+switch_active_page(GtkNotebook *notebook, GwyParams *params)
+{
+    gint id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(notebook), "gwy-active-page-param-id"));
+    gtk_notebook_set_current_page(notebook, gwy_params_get_int(params, id));
+}
+
+/**
+ * gwy_param_active_page_link_to_notebook:
+ * @params: A parameter value set.
+ * @id: Parameter identifier.
+ * @notebook: A notebook widget.
+ *
+ * Connects an active page parameter to a notebook.
+ *
+ * This function should be called after the notebook was fully constructed.  The active notebook page is set according
+ * to the parameter (clamping values that refer to non-existent pages).  Future page switches will update the
+ * parameter value, allowing to simply save it to settings when the module dialog is finished.
+ *
+ * Since: 2.59
+ **/
+void
+gwy_param_active_page_link_to_notebook(GwyParams *params,
+                                       gint id,
+                                       GtkNotebook *notebook)
+{
+    GwyParamDef *pardef;
+    const GwyParamDefItem *def;
+    gint pageno, npages;
+
+    g_return_if_fail(GWY_IS_PARAMS(params));
+    g_return_if_fail(GTK_IS_NOTEBOOK(notebook));
+    g_return_if_fail(!g_object_get_data(G_OBJECT(notebook), "gwy-active-page-param-id"));
+
+    pardef = gwy_params_get_def(params);
+    def = _gwy_param_def_item(pardef, _gwy_param_def_index(pardef, id));
+    g_return_if_fail(def && def->type == GWY_PARAM_ACTIVE_PAGE);
+
+    npages = gtk_notebook_get_n_pages(notebook);
+    pageno = gwy_params_get_int(params, id);
+    pageno = CLAMP(pageno, 0, npages-1);
+    gwy_params_set_int(params, id, pageno);
+
+    /* Here we assume params will outlive notebook. */
+    g_object_set_data(G_OBJECT(notebook), "gwy-active-page-param-id", GINT_TO_POINTER(id));
+    /* NB: Switching pages before the notebook is shown has no effect.  See GtkNotebook docs. */
+    g_signal_connect(notebook, "show", G_CALLBACK(switch_active_page), params);
+    g_signal_connect(notebook, "switch-page", G_CALLBACK(active_page_changed), params);
 }
 
 /************************** Documentation ****************************/
@@ -813,11 +1076,9 @@ gwy_preview_surface_to_datafield(GwySurface *surface,
 
 /**
  * GwySaveAuxiliaryCreate:
- * @user_data: The data passed to gwy_save_auxiliary_with_callback() as
- *             @user_data.
- * @data_len: The length of the returned data in bytes.  Leaving it unset has
- *            the same effect as setting it to a negative value.  See
- *            gwy_save_auxiliary_data() for details.
+ * @user_data: The data passed to gwy_save_auxiliary_with_callback() as @user_data.
+ * @data_len: The length of the returned data in bytes.  Leaving it unset has the same effect as setting it to
+ *            a negative value.  See gwy_save_auxiliary_data() for details.
  *
  * The type of auxiliary saved data creation function.
  *
@@ -828,10 +1089,8 @@ gwy_preview_surface_to_datafield(GwySurface *surface,
 
 /**
  * GwySaveAuxiliaryDestroy:
- * @data: The data returned by the corresponding #GwySaveAuxiliaryCreate
- *        function.
- * @user_data: The data passed to gwy_save_auxiliary_with_callback() as
- *             @user_data.
+ * @data: The data returned by the corresponding #GwySaveAuxiliaryCreate function.
+ * @user_data: The data passed to gwy_save_auxiliary_with_callback() as @user_data.
  *
  * The type of auxiliary saved data destruction function.
  *
@@ -840,12 +1099,9 @@ gwy_preview_surface_to_datafield(GwySurface *surface,
 
 /**
  * GwyPreviewSurfaceFlags:
- * @GWY_PREVIEW_SURFACE_DENSITY: Render a point density map instead of the
- *                               data.
- * @GWY_PREVIEW_SURFACE_FILL: Make the data field as large as the specified
- *                            resolutions at least in one dimension (it can
- *                            be prevented in the other by different aspect
- *                            ratio).
+ * @GWY_PREVIEW_SURFACE_DENSITY: Render a point density map instead of the data.
+ * @GWY_PREVIEW_SURFACE_FILL: Make the data field as large as the specified resolutions at least in one dimension (it
+ *                            can be prevented in the other by different aspect ratio).
  *
  * Type of behaviour modifying flags that can be passed to
  * gwy_preview_surface_to_datafield().
@@ -853,4 +1109,4 @@ gwy_preview_surface_to_datafield(GwySurface *surface,
  * Since: 2.46
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin et columns=120 tw=118 ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
