@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyresultsexport.c 21852 2019-01-24 10:27:06Z yeti-dn $
+ *  $Id: gwyresultsexport.c 23208 2021-03-08 15:55:47Z yeti-dn $
  *  Copyright (C) 2017-2019 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -510,15 +510,8 @@ void
 gwy_results_export_set_title(GwyResultsExport *rexport,
                              const gchar *title)
 {
-    gchar *old;
-
     g_return_if_fail(GWY_IS_RESULTS_EXPORT(rexport));
-    if (title == rexport->priv->title)
-        return;
-
-    old = rexport->priv->title;
-    rexport->priv->title = g_strdup(title);
-    g_free(old);
+    gwy_assign_string(&rexport->priv->title, title);
 }
 
 /**
