@@ -1,5 +1,5 @@
 /*
- *  $Id: log.c 22982 2020-11-30 15:59:39Z yeti-dn $
+ *  $Id: log.c 23422 2021-04-01 14:36:18Z yeti-dn $
  *  Copyright (C) 2014-2016 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -345,7 +345,7 @@ data_log_add_valist(GwyContainer *data,
     else
         targetlog = get_data_log(data, newquark, FALSE);
 
-    if (targetlog && targetlog != sourcelog) {
+    if (targetlog && sourcelog && targetlog != sourcelog) {
         g_warning("Target log must not exist when replicating logs.");
         /* Fix the operation to simple log-append. */
         sourcelog = targetlog;
