@@ -1,5 +1,5 @@
 /*
- *  $Id: menu.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: menu.c 23078 2021-01-23 09:08:43Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -212,6 +212,7 @@ gwy_app_menu_resolve_translations(GNode *node,
     MenuNodeData *data = (MenuNodeData*)node->data;
     MenuNodeData *pdata;
     const gchar *p;
+    gchar *s;
 
     if (G_NODE_IS_ROOT(node))
         return FALSE;
@@ -230,8 +231,15 @@ gwy_app_menu_resolve_translations(GNode *node,
     data->item_translated = g_strdup(p+1);
     data->item_translated_canonical = g_strdup(data->item_translated);
     gwy_app_menu_canonicalize_label(data->item_translated_canonical);
-    data->item_collated = g_utf8_collate_key(data->item_translated_canonical,
-                                             -1);
+    /* This seems silly but I am seeing identical collation keys for different
+     * strings (gwy_app_menu_sort_submenus prints a warning).  Perhaps it only
+     * happens when translation language is selected by LANGUAGE env variable
+     * but all other locales are different (g_utf8_colllate() is
+     * locale-dependent). Anyway, it should not disrupt the order. */
+    s = g_utf8_collate_key(data->item_translated_canonical, -1);
+    data->item_collated = g_strconcat(s, " ", data->item_translated_canonical,
+                                      NULL);
+    g_free(s);
 
     if (!pdata->path_translated) {
         pdata->path_translated = g_strndup(data->path_translated,
