@@ -1,5 +1,5 @@
 /*
- *  $Id: meta.c 20678 2017-12-18 18:26:55Z yeti-dn $
+ *  $Id: meta.c 23208 2021-03-08 15:55:47Z yeti-dn $
  *  Copyright (C) 2003-2013 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -488,8 +488,7 @@ gwy_meta_cell_edited(GtkCellRendererText *renderer,
 
         case META_VALUE:
         if (pango_parse_markup(text, -1, 0, NULL, NULL, NULL, NULL))
-            gwy_container_set_string(browser->meta, oldkey,
-                                     g_strdup(text));
+            gwy_container_set_string(browser->meta, oldkey, g_strdup(text));
         break;
 
         default:
