@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyenum.c 21692 2018-11-26 13:29:49Z yeti-dn $
+ *  $Id: gwyenum.c 23787 2021-05-26 12:13:36Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -47,7 +47,7 @@ gwy_enum_get_type(void)
  * @str: A string containing one of @enum_table string values.
  * @enum_table: A table of corresponding string-integer pairs.
  * @n: The number of elements in @enum_table, may be -1 when @enum_table is
- *     terminated by a #NULL name.
+ *     terminated by a %NULL name.
  *
  * Creates an integer representation of a string enum value @str.
  *
@@ -74,7 +74,7 @@ gwy_string_to_enum(const gchar *str,
  * @enumval: A one integer value from @enum_table.
  * @enum_table: A table of corresponding string-integer pairs.
  * @n: The number of elements in @enum_table, may be -1 when @enum_table is
- *     terminated by a #NULL name.
+ *     terminated by a %NULL name.
  *
  * Creates a string representation of an integer enum value @enumval.
  *
@@ -136,8 +136,8 @@ gwy_enuml_to_string(gint enumval,
  * @str: A string containing one of @enum_table string values.
  * @enum_table: A table of corresponding string-integer pairs.
  * @n: The number of elements in @enum_table, may be -1 when @enum_table is
- *     terminated by a #NULL name.
- * @delimiter: A delimiter to split @str on, when #NULL space is used.
+ *     terminated by a %NULL name.
+ * @delimiter: A delimiter to split @str on, when %NULL space is used.
  *
  * Creates an integer flag combination of its string representation @str.
  *
@@ -177,8 +177,8 @@ gwy_string_to_flags(const gchar *str,
  * @enumval: Some ORed integer flags from @enum_table.
  * @enum_table: A table of corresponding string-integer pairs.
  * @n: The number of elements in @enum_table, may be -1 when @enum_table is
- *     terminated by a #NULL name.
- * @glue: A glue to join string values with, when #NULL space is used.
+ *     terminated by a %NULL name.
+ * @glue: A glue to join string values with, when %NULL space is used.
  *
  * Creates a string representation of integer flag combination @enumval.
  *
@@ -368,6 +368,97 @@ gwy_enum_inventory_new(const GwyEnum *enum_table,
                                         n, enum_table);
 }
 
+/**
+ * gwy_enum_fill_from_struct:
+ * @enum_table: A table of string-integer pairs to fill, possibly %NULL to allocate a new one.
+ * @n: The number of elements in @enum_table.  It must not be -1.
+ * @items: Struct items containing the names and values at given offsets.
+ * @item_size: Size of one item in @items, in bytes.
+ * @name_offset: Offset of the name field (of gchar* type) in the struct, in bytes.
+ * @value_offset: Offset of the value field (of 32bit integer type) in the struct, in bytes.
+ *                Pass -1 if values are just item indices.
+ *
+ * Fills an enum definition from another struct.
+ *
+ * Use G_STRUCT_OFFSET() to obtain @name_offset and @value_offset.
+ *
+ * If @value_offset is -1 the enum values are made equal to @items array
+ * indices from 0 to @n-1.  Otherwise they values are taken from @items and can be arbitrary.
+ *
+ * If @enum_table is not %NULL and its value fields are filled (i.e. not all
+ * zeros), the function just fills the corresponding names according to @items.  This allows
+ * reordering @enum_table with respect to @items.
+ *
+ * In all other cases both names and values are filled in the same order as in @items.
+ *
+ * The string names are not duplicated; they are only copied as pointers.
+ *
+ * Returns: Either @enum_table itself, or a newly allocated enum table if it was %NULL.
+ *
+ * Since: 2.59
+ **/
+GwyEnum*
+gwy_enum_fill_from_struct(GwyEnum *enum_table,
+                          gint n,
+                          gconstpointer items,
+                          guint item_size,
+                          gint name_offset,
+                          gint value_offset)
+{
+    gboolean table_is_prefilled = FALSE, struct_has_values = (value_offset >= 0);
+    gconstpointer item;
+    gint i, j, v;
+
+    if (!n)
+        return enum_table;
+
+    g_return_val_if_fail(items, enum_table);
+    g_return_val_if_fail(name_offset >= 0, enum_table);
+
+    if (!enum_table)
+        enum_table = g_new0(GwyEnum, n);
+    else {
+        for (i = 0; i < n; i++) {
+            if (enum_table[i].value != 0) {
+                table_is_prefilled = TRUE;
+                break;
+            }
+        }
+    }
+
+    for (i = 0; i < n; i++) {
+        if (table_is_prefilled) {
+            v = enum_table[i].value;
+            if (struct_has_values) {
+                for (j = 0; i < n; j++) {
+                    item = (gconstpointer)((const gchar*)items + j*item_size);
+                    if (G_STRUCT_MEMBER(gint32, item, value_offset) == v)
+                        break;
+                }
+                if (j == n) {
+                    g_warning("Cannot find pre-filled enum value %d in struct items.", v);
+                    item = items;
+                }
+            }
+            else {
+                if (v < 0 || v >= n) {
+                    g_warning("Value %d is out of range 0..%d and cannot be used as index.", v, n);
+                    v = 0;
+                }
+                item = (gconstpointer)((const gchar*)items + v*item_size);
+            }
+        }
+        else {
+            item = (gconstpointer)((const gchar*)items + i*item_size);
+            v = (struct_has_values ? G_STRUCT_MEMBER(gint32, item, value_offset) : i);
+        }
+        enum_table[i].value = v;
+        enum_table[i].name = G_STRUCT_MEMBER(gchar*, item, name_offset);
+    }
+
+    return enum_table;
+}
+
 /************************** Documentation ****************************/
 
 /**
