@@ -1,5 +1,5 @@
 /*
- *  $Id: gwysiunit.c 22807 2020-05-07 11:50:03Z yeti-dn $
+ *  $Id: gwysiunit.c 23851 2021-06-14 16:33:38Z yeti-dn $
  *  Copyright (C) 2004-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1040,7 +1040,7 @@ gwy_si_unit_parse(GwySIUnit *siunit,
 
         /* get unit power */
         unit.power = 1;
-        if ((p = strstr(buf->str + 1, "<sup>"))) {
+        if (buf->len && (p = strstr(buf->str + 1, "<sup>"))) {
             unit.power = strtol(p + strlen("<sup>"), &e, 10);
             if (e == p + strlen("<sup>")
                 || !g_str_has_prefix(e, "</sup>")) {
@@ -1053,7 +1053,7 @@ gwy_si_unit_parse(GwySIUnit *siunit,
             }
             g_string_truncate(buf, p - buf->str);
         }
-        else if ((p = strchr(buf->str + 1, '^'))) {
+        else if (buf->len && (p = strchr(buf->str + 1, '^'))) {
             unit.power = strtol(p + 1, &e, 10);
             if (e == p + 1 || *e) {
                 /* g_warning("Bad power %s", p); */
