@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyutils.c 22669 2019-11-10 10:40:40Z yeti-dn $
+ *  $Id: gwyutils.c 23201 2021-03-07 11:54:21Z yeti-dn $
  *  Copyright (C) 2003-2019 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1633,6 +1633,48 @@ gwy_raw_data_size(GwyRawDataType datatype)
     return sizes[datatype];
 }
 
+/**
+ * gwy_assign_string:
+ * @target: Pointer to target string, typically a struct field.
+ * @newvalue: New value of the string, may be %NULL.
+ *
+ * Assigns a string, checking for equality and handling %NULL<!-- -->s.
+ *
+ * This function simplifies handling of string value setters.
+ *
+ * The new value is duplicated and the old string is freed in a safe manner
+ * (it is possible to pass a pointer somewhere within the old value as the
+ * new value, for instance).  Any of the old and new value can be %NULL.  If
+ * both values are equal (including both unset), the function returns %FALSE.
+ *
+ * Returns: %TRUE if the target string has changed.
+ *
+ * Since: 2.59
+ **/
+gboolean
+gwy_assign_string(gchar **target,
+                  const gchar *newvalue)
+{
+    if (*target && newvalue) {
+        if (!gwy_strequal(*target, newvalue)) {
+            gchar *old = *target;
+            *target = g_strdup(newvalue);
+            g_free(old);
+            return TRUE;
+        }
+    }
+    else if (*target) {
+        g_free(*target);
+        *target = NULL;
+        return TRUE;
+    }
+    else if (newvalue) {
+        *target = g_strdup(newvalue);
+        return TRUE;
+    }
+    return FALSE;
+}
+
 /**
  * gwy_object_set_or_reset:
  * @object: A #GObject.
