@@ -1,5 +1,5 @@
 /*
- *  $Id: gwyutils.h 21366 2018-08-29 12:31:19Z yeti-dn $
+ *  $Id: gwyutils.h 23201 2021-03-07 11:54:21Z yeti-dn $
  *  Copyright (C) 2003-2011 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -111,6 +111,8 @@ void         gwy_convert_raw_data      (gconstpointer data,
                                         gdouble scale,
                                         gdouble offset);
 guint        gwy_raw_data_size         (GwyRawDataType datatype);
+gboolean     gwy_assign_string         (gchar **target,
+                                        const gchar *newvalue);
 void         gwy_object_set_or_reset   (gpointer object,
                                         GType type,
                                         ...)                        G_GNUC_NULL_TERMINATED;
