@@ -1,22 +1,18 @@
 /*
- *  $Id: arithmetic.c 22335 2019-07-25 08:23:38Z yeti-dn $
- *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
+ *  $Id: arithmetic.c 23659 2021-05-10 17:29:09Z yeti-dn $
+ *  Copyright (C) 2003-2021 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public
+ *  License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any
+ *  later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ *  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -37,17 +33,14 @@ compatibility_check_common(GwyDataField *result,
 {
     GwyDataCompatibilityFlags flags = GWY_DATA_COMPATIBILITY_RES;
     g_return_val_if_fail(GWY_IS_DATA_FIELD(result), FALSE);
-    g_return_val_if_fail(!gwy_data_field_check_compatibility(result, operand1,
-                                                             flags), FALSE);
-    g_return_val_if_fail(!gwy_data_field_check_compatibility(result, operand2,
-                                                             flags), FALSE);
+    g_return_val_if_fail(!gwy_data_field_check_compatibility(result, operand1, flags), FALSE);
+    g_return_val_if_fail(!gwy_data_field_check_compatibility(result, operand2, flags), FALSE);
     return TRUE;
 }
 
 /**
  * gwy_data_field_sum_fields:
- * @result: A data field to put the result to.  May be one of @operand1,
- *          @operand2.
+ * @result: A data field to put the result to.  May be one of @operand1, @operand2.
  * @operand1: First data field operand.
  * @operand2: Second data field operand.
  *
@@ -82,8 +75,7 @@ gwy_data_field_sum_fields(GwyDataField *result,
 
 /**
  * gwy_data_field_subtract_fields:
- * @result: A data field to put the result to.  May be one of @operand1,
- *          @operand2.
+ * @result: A data field to put the result to.  May be one of @operand1, @operand2.
  * @operand1: First data field operand.
  * @operand2: Second data field operand.
  *
@@ -118,8 +110,7 @@ gwy_data_field_subtract_fields(GwyDataField *result,
 
 /**
  * gwy_data_field_multiply_fields:
- * @result: A data field to put the result to.  May be one of @operand1,
- *          @operand2.
+ * @result: A data field to put the result to.  May be one of @operand1, @operand2.
  * @operand1: First data field operand.
  * @operand2: Second data field operand.
  *
@@ -149,8 +140,7 @@ gwy_data_field_multiply_fields(GwyDataField *result,
 
 /**
  * gwy_data_field_divide_fields:
- * @result: A data field to put the result to.  May be one of @operand1,
- *          @operand2.
+ * @result: A data field to put the result to.  May be one of @operand1, @operand2.
  * @operand1: First data field operand.
  * @operand2: Second data field operand.
  *
@@ -180,8 +170,7 @@ gwy_data_field_divide_fields(GwyDataField *result,
 
 /**
  * gwy_data_field_min_of_fields:
- * @result: A data field to put the result to.  May be one of @operand1,
- *          @operand2.
+ * @result: A data field to put the result to.  May be one of @operand1, @operand2.
  * @operand1: First data field operand.
  * @operand2: Second data field operand.
  *
@@ -216,8 +205,7 @@ gwy_data_field_min_of_fields(GwyDataField *result,
 
 /**
  * gwy_data_field_max_of_fields:
- * @result: A data field to put the result to.  May be one of @operand1,
- *          @operand2.
+ * @result: A data field to put the result to.  May be one of @operand1, @operand2.
  * @operand1: First data field operand.
  * @operand2: Second data field operand.
  *
@@ -252,8 +240,7 @@ gwy_data_field_max_of_fields(GwyDataField *result,
 
 /**
  * gwy_data_field_hypot_of_fields:
- * @result: A data field to put the result to.  May be one of @operand1,
- *          @operand2.
+ * @result: A data field to put the result to.  May be one of @operand1, @operand2.
  * @operand1: First data field operand.
  * @operand2: Second data field operand.
  *
@@ -287,6 +274,49 @@ gwy_data_field_hypot_of_fields(GwyDataField *result,
     gwy_data_field_invalidate(result);
 }
 
+/**
+ * gwy_data_field_linear_combination:
+ * @result: A data field to put the result to.  May be one of @operand1, @operand2.
+ * @constant: Constant term to add to the result.
+ * @operand1: First data field operand.
+ * @coeff1: Factor to multiply the first operand with.
+ * @operand2: Second data field operand.
+ * @coeff2: Factor to multiply the second operand with.
+ *
+ * Computes point-wise general linear combination of two data fields.
+ *
+ * Since: 2.59
+ **/
+void
+gwy_data_field_linear_combination(GwyDataField *result,
+                                  gdouble coeff1,
+                                  GwyDataField *operand1,
+                                  gdouble coeff2,
+                                  GwyDataField *operand2,
+                                  gdouble constant)
+{
+    gdouble *p, *q, *r;
+    gint n, i;
+
+    if (!compatibility_check_common(result, operand1, operand2))
+        return;
+
+    r = result->data;
+    p = operand1->data;
+    q = operand2->data;
+    n = result->xres * result->yres;
+    /* Too trivial to parallelise? */
+#ifdef _OPENMP
+#pragma omp parallel for if(gwy_threads_are_enabled()) default(none) \
+            private(i) \
+            shared(r,p,q,n,constant,coeff1,coeff2)
+#endif
+    for (i = 0; i < n; i++)
+        r[i] = coeff1*p[i] + coeff2*q[i] + constant;
+
+    gwy_data_field_invalidate(result);
+}
+
 static void
 check_basic_properties(gint res1, gint res2,
                        gdouble real1, gdouble real2,
@@ -294,15 +324,13 @@ check_basic_properties(gint res1, gint res2,
                        GwyDataCompatibilityFlags *result)
 {
     /* Resolution */
-    if ((check & GWY_DATA_COMPATIBILITY_RES)
-        && !(*result & GWY_DATA_COMPATIBILITY_RES)) {
+    if ((check & GWY_DATA_COMPATIBILITY_RES) && !(*result & GWY_DATA_COMPATIBILITY_RES)) {
         if (res1 != res2)
             *result |= GWY_DATA_COMPATIBILITY_RES;
     }
 
     /* Real size */
-    if (check & GWY_DATA_COMPATIBILITY_REAL
-        && !(*result & GWY_DATA_COMPATIBILITY_REAL)) {
+    if (check & GWY_DATA_COMPATIBILITY_REAL && !(*result & GWY_DATA_COMPATIBILITY_REAL)) {
         /* Keeps the condition in negative form to catch NaNs and odd values
          * as incompatible. */
         if (!(fabs(log(real1/real2)) <= EPSILON))
@@ -310,8 +338,7 @@ check_basic_properties(gint res1, gint res2,
     }
 
     /* Measure */
-    if (check & GWY_DATA_COMPATIBILITY_MEASURE
-        && !(*result & GWY_DATA_COMPATIBILITY_MEASURE)) {
+    if (check & GWY_DATA_COMPATIBILITY_MEASURE && !(*result & GWY_DATA_COMPATIBILITY_MEASURE)) {
         if (!(fabs(log(real1/res1*res2/real2)) <= EPSILON))
             *result |= GWY_DATA_COMPATIBILITY_MEASURE;
     }
@@ -339,8 +366,8 @@ units_are_equal(GwySIUnit *unit1, GwySIUnit *unit2)
  *
  * Checks whether two data fields are compatible.
  *
- * Returns: Zero if all tested properties are compatible.  Flags corresponding
- *          to failed tests if data fields are not compatible.
+ * Returns: Zero if all tested properties are compatible.  Flags corresponding to failed tests if data fields are not
+ *          compatible.
  **/
 GwyDataCompatibilityFlags
 gwy_data_field_check_compatibility(GwyDataField *data_field1,
@@ -352,19 +379,15 @@ gwy_data_field_check_compatibility(GwyDataField *data_field1,
     g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field1), check);
     g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field2), check);
 
-    check_basic_properties(data_field1->xres, data_field2->xres,
-                           data_field1->xreal, data_field2->xreal,
+    check_basic_properties(data_field1->xres, data_field2->xres, data_field1->xreal, data_field2->xreal,
                            check, &result);
-    check_basic_properties(data_field1->yres, data_field2->yres,
-                           data_field1->yreal, data_field2->yreal,
+    check_basic_properties(data_field1->yres, data_field2->yres, data_field1->yreal, data_field2->yreal,
                            check, &result);
 
-    if ((check & GWY_DATA_COMPATIBILITY_LATERAL)
-        && !units_are_equal(data_field1->si_unit_xy, data_field2->si_unit_xy))
+    if ((check & GWY_DATA_COMPATIBILITY_LATERAL) && !units_are_equal(data_field1->si_unit_xy, data_field2->si_unit_xy))
         result |= GWY_DATA_COMPATIBILITY_LATERAL;
 
-    if ((check & GWY_DATA_COMPATIBILITY_VALUE)
-        && !units_are_equal(data_field1->si_unit_z, data_field2->si_unit_z))
+    if ((check & GWY_DATA_COMPATIBILITY_VALUE) && !units_are_equal(data_field1->si_unit_z, data_field2->si_unit_z))
         result |= GWY_DATA_COMPATIBILITY_VALUE;
 
     return result;
@@ -378,8 +401,8 @@ gwy_data_field_check_compatibility(GwyDataField *data_field1,
  *
  * Checks whether two data lines are compatible.
  *
- * Returns: Zero if all tested properties are compatible.  Flags corresponding
- *          to failed tests if data lines are not compatible.
+ * Returns: Zero if all tested properties are compatible.  Flags corresponding to failed tests if data lines are not
+ *          compatible.
  **/
 GwyDataCompatibilityFlags
 gwy_data_line_check_compatibility(GwyDataLine *data_line1,
@@ -391,16 +414,12 @@ gwy_data_line_check_compatibility(GwyDataLine *data_line1,
     g_return_val_if_fail(GWY_IS_DATA_LINE(data_line1), check);
     g_return_val_if_fail(GWY_IS_DATA_LINE(data_line2), check);
 
-    check_basic_properties(data_line1->res, data_line2->res,
-                           data_line1->real, data_line2->real,
-                           check, &result);
+    check_basic_properties(data_line1->res, data_line2->res, data_line1->real, data_line2->real, check, &result);
 
-    if ((check & GWY_DATA_COMPATIBILITY_LATERAL)
-        && !units_are_equal(data_line1->si_unit_x, data_line2->si_unit_x))
+    if ((check & GWY_DATA_COMPATIBILITY_LATERAL) && !units_are_equal(data_line1->si_unit_x, data_line2->si_unit_x))
         result |= GWY_DATA_COMPATIBILITY_LATERAL;
 
-    if ((check & GWY_DATA_COMPATIBILITY_VALUE)
-        && !units_are_equal(data_line1->si_unit_y, data_line2->si_unit_y))
+    if ((check & GWY_DATA_COMPATIBILITY_VALUE) && !units_are_equal(data_line1->si_unit_y, data_line2->si_unit_y))
         result |= GWY_DATA_COMPATIBILITY_VALUE;
 
     return result;
@@ -414,8 +433,7 @@ data_lines_equal(GwyDataLine *data_line1,
     const gdouble *data1, *data2;
 
     if (gwy_data_line_check_compatibility(data_line1, data_line2,
-                                          GWY_DATA_COMPATIBILITY_RES
-                                          | GWY_DATA_COMPATIBILITY_VALUE))
+                                          GWY_DATA_COMPATIBILITY_RES | GWY_DATA_COMPATIBILITY_VALUE))
         return FALSE;
 
     res = data_line1->res;
@@ -438,11 +456,11 @@ data_lines_equal(GwyDataLine *data_line1,
  *
  * Checks whether two data bricks are compatible.
  *
- * Real dimensions are checked without regard to calibration.  Calibrations are
- * considered compatible if either both exist and are identical or none exists.
+ * Real dimensions are checked without regard to calibration.  Calibrations are considered compatible if either both
+ * exist and are identical or none exists.
  *
- * Returns: Zero if all tested properties are compatible.  Flags corresponding
- *          to failed tests if bricks are not compatible.
+ * Returns: Zero if all tested properties are compatible.  Flags corresponding to failed tests if bricks are not
+ *          compatible.
  *
  * Since: 2.51
  **/
@@ -497,8 +515,8 @@ gwy_brick_check_compatibility(GwyBrick *brick1,
  *
  * Checks whether a data field is compatible with brick XY-planes.
  *
- * Returns: Zero if all tested properties are compatible.  Flags corresponding
- *          to failed tests if the data objects are not compatible.
+ * Returns: Zero if all tested properties are compatible.  Flags corresponding to failed tests if the data objects are
+ *          not compatible.
  *
  * Since: 2.51
  **/
@@ -512,12 +530,8 @@ gwy_data_field_check_compatibility_with_brick_xy(GwyDataField *data_field,
     g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), check);
     g_return_val_if_fail(GWY_IS_BRICK(brick), check);
 
-    check_basic_properties(data_field->xres, brick->xres,
-                           data_field->xreal, brick->xreal,
-                           check, &result);
-    check_basic_properties(data_field->yres, brick->yres,
-                           data_field->yreal, brick->yreal,
-                           check, &result);
+    check_basic_properties(data_field->xres, brick->xres, data_field->xreal, brick->xreal, check, &result);
+    check_basic_properties(data_field->yres, brick->yres, data_field->yreal, brick->yreal, check, &result);
 
     if ((check & GWY_DATA_COMPATIBILITY_LATERAL)
         && (!units_are_equal(data_field->si_unit_xy, brick->si_unit_x)
@@ -539,16 +553,14 @@ gwy_data_field_check_compatibility_with_brick_xy(GwyDataField *data_field,
  *
  * Checks whether a data line is compatible with brick Z-profiles.
  *
- * If @check includes %GWY_DATA_COMPATIBILITY_REAL or
- * %GWY_DATA_COMPATIBILITY_LATERAL but not %GWY_DATA_COMPATIBILITY_AXISCAL,
- * @data_line is simply compared to @brick in the Z direction.
+ * If @check includes %GWY_DATA_COMPATIBILITY_REAL or %GWY_DATA_COMPATIBILITY_LATERAL but not
+ * %GWY_DATA_COMPATIBILITY_AXISCAL, @data_line is simply compared to @brick in the Z direction.
  *
- * If you include %GWY_DATA_COMPATIBILITY_AXISCAL and @brick has a
- * Z-calibration data line, then the value range and units of this data line
- * are compared to @data_line.  This may not be very useful.
+ * If you include %GWY_DATA_COMPATIBILITY_AXISCAL and @brick has a Z-calibration data line, then the value range and
+ * units of this data line are compared to @data_line.  This may not be very useful.
  *
- * Returns: Zero if all tested properties are compatible.  Flags corresponding
- *          to failed tests if the data objects are not compatible.
+ * Returns: Zero if all tested properties are compatible.  Flags corresponding to failed tests if the data objects are
+ *          not compatible.
  *
  * Since: 2.51
  **/
@@ -563,19 +575,15 @@ gwy_data_line_check_compatibility_with_brick_z(GwyDataLine *data_line,
     g_return_val_if_fail(GWY_IS_DATA_LINE(data_line), check);
     g_return_val_if_fail(GWY_IS_BRICK(brick), check);
 
-    if (check & GWY_DATA_COMPATIBILITY_VALUE
-        && !units_are_equal(data_line->si_unit_y, brick->si_unit_w))
+    if (check & GWY_DATA_COMPATIBILITY_VALUE && !units_are_equal(data_line->si_unit_y, brick->si_unit_w))
         result |= GWY_DATA_COMPATIBILITY_VALUE;
 
     /* Without zcalibration compare directly to the brick. */
     zcal = gwy_brick_get_zcalibration(brick);
     if (!zcal || !(check & GWY_DATA_COMPATIBILITY_AXISCAL)) {
-        check_basic_properties(data_line->res, brick->xres,
-                               data_line->real, brick->xreal,
-                               check, &result);
+        check_basic_properties(data_line->res, brick->xres, data_line->real, brick->xreal, check, &result);
 
-        if ((check & GWY_DATA_COMPATIBILITY_LATERAL)
-            && !units_are_equal(data_line->si_unit_x, brick->si_unit_z))
+        if ((check & GWY_DATA_COMPATIBILITY_LATERAL) && !units_are_equal(data_line->si_unit_x, brick->si_unit_z))
             result |= GWY_DATA_COMPATIBILITY_LATERAL;
 
         return result;
@@ -585,13 +593,10 @@ gwy_data_line_check_compatibility_with_brick_z(GwyDataLine *data_line,
      * same thing as *coordinates* of @data_line.  So this is a mess and
      * possibly not useful at all. */
     g_assert(zcal->res == brick->zres);
-    check_basic_properties(data_line->res, zcal->res,
-                           data_line->real,
-                           zcal->data[zcal->res-1] - zcal->data[0],
+    check_basic_properties(data_line->res, zcal->res, data_line->real, zcal->data[zcal->res-1] - zcal->data[0],
                            check, &result);
 
-    if ((check & GWY_DATA_COMPATIBILITY_LATERAL)
-        && !units_are_equal(data_line->si_unit_x, zcal->si_unit_y))
+    if ((check & GWY_DATA_COMPATIBILITY_LATERAL) && !units_are_equal(data_line->si_unit_x, zcal->si_unit_y))
         result |= GWY_DATA_COMPATIBILITY_LATERAL;
 
     return result;
@@ -929,11 +934,8 @@ rect_extend_laplace(const gdouble *in, guint inrowstride,
 
     if (extend_down + extend_up + extend_right + extend_left == 0) {
         /* Direct copy of the ROI */
-        for (i = 0; i < height; i++) {
-            gwy_assign(out + (extend_up + i)*outrowstride + extend_left,
-                       in + (ypos + i)*inrowstride + xpos,
-                       width);
-        }
+        for (i = 0; i < height; i++)
+            gwy_assign(out + (extend_up + i)*outrowstride + extend_left, in + (ypos + i)*inrowstride + xpos, width);
         return;
     }
 
@@ -947,17 +949,11 @@ rect_extend_laplace(const gdouble *in, guint inrowstride,
      * rowstride than extxres! */
     workspace = gwy_data_field_new(extxres, extyres, 1.0, 1.0, FALSE);
 
-    for (i = 0; i < height; i++) {
-        gwy_assign(workspace->data + (extend_up + i)*extxres + extend_left,
-                   in + (ypos + i)*inrowstride + xpos,
-                   width);
-    }
+    for (i = 0; i < height; i++)
+        gwy_assign(workspace->data + (extend_up + i)*extxres + extend_left, in + (ypos + i)*inrowstride + xpos, width);
     gwy_data_field_laplace_solve(workspace, mask, -1, 0.5);
-    for (i = 0; i < extyres; i++) {
-        gwy_assign(out + i*outrowstride,
-                   workspace->data + i*extxres,
-                   extxres);
-    }
+    for (i = 0; i < extyres; i++)
+        gwy_assign(out + i*outrowstride, workspace->data + i*extxres, extxres);
 
     g_object_unref(workspace);
     g_object_unref(mask);
@@ -997,18 +993,15 @@ _gwy_get_rect_extend_func(GwyExteriorType exterior)
  * gwy_data_field_extend:
  * @data_field: A two-dimensional data field.
  * @left: Number of pixels to extend to the left (towards lower column indices).
- * @right: Number of pixels to extend to the right (towards higher column
- *         indices).
+ * @right: Number of pixels to extend to the right (towards higher column indices).
  * @up: Number of pixels to extend up (towards lower row indices).
  * @down: Number of pixels to extend down (towards higher row indices).
  * @exterior: Exterior pixels handling.
  * @fill_value: The value to use with %GWY_EXTERIOR_FIXED_VALUE exterior.
- * @keep_offsets: %TRUE to set the X and Y offsets of the new field
- *                using @field offsets.  %FALSE to set offsets
- *                of the new field to zeroes.
+ * @keep_offsets: %TRUE to set the X and Y offsets of the new field using @field offsets.  %FALSE to set offsets of
+ *                the new field to zeroes.
  *
- * Creates a new data field by extending another data field using the specified
- * method of exterior handling.
+ * Creates a new data field by extending another data field using the specified method of exterior handling.
  *
  * Returns: A newly created data field.
  *
@@ -1034,8 +1027,7 @@ gwy_data_field_extend(GwyDataField *data_field,
 
     width = data_field->xres;
     height = data_field->yres;
-    target = gwy_data_field_new(width + left + right, height + up + down,
-                                1.0, 1.0, FALSE);
+    target = gwy_data_field_new(width + left + right, height + up + down, 1.0, 1.0, FALSE);
     extend_rect(data_field->data, data_field->xres, target->data, target->xres,
                 col, row, width, height, data_field->xres, data_field->yres,
                 left, right, up, down, fill_value);
@@ -1064,16 +1056,12 @@ gwy_data_field_extend(GwyDataField *data_field,
  * @title: arithmetic
  * @short_description: Arithmetic opetations on data fields
  *
- * Data arithmetic functions perform simple operations combining several data
- * fields.  Their sizes have to be size-compatible, i.e.
- * gwy_data_field_check_compatibility(operand1, operand2,
- * GWY_DATA_COMPATIBILITY_RES)
- * must pass and the same must hold for the data field to store the result to.
+ * Data arithmetic functions perform simple operations combining several data fields.  Their sizes have to be
+ * size-compatible, i.e. gwy_data_field_check_compatibility(operand1, operand2, GWY_DATA_COMPATIBILITY_RES) must pass
+ * and the same must hold for the data field to store the result to.
  *
- * Functions gwy_data_field_check_compatibility(),
- * gwy_data_line_check_compatibility() and gwy_brick_check_compatibility()
- * simplify testing compatibility of data fields, lines and bricks,
- * respectively.
+ * Functions gwy_data_field_check_compatibility(), gwy_data_line_check_compatibility() and
+ * gwy_brick_check_compatibility() simplify testing compatibility of data fields, lines and bricks, respectively.
  **/
 
-/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
+/* vim: set cin columns=120 tw=118 et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
