@@ -1,5 +1,5 @@
 /*
- *  $Id: correlation.c 22486 2019-09-16 16:32:19Z yeti-dn $
+ *  $Id: correlation.c 23577 2021-04-25 22:00:26Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -972,8 +972,8 @@ gwy_data_field_crosscorrelate_iteration(GwyComputationState *cstate)
                                                       state->weights,
                                                       col - state->window_width/2,
                                                       row - state->window_height/2,
-                                                      colmax - state->window_width/2 + n,
-                                                      rowmax - state->window_height/2 + m,
+                                                      CLAMP(colmax - state->window_width/2 + n, 0, xres-1),
+                                                      CLAMP(rowmax - state->window_height/2 + m, 0, yres-1),
                                                       state->window_width,
                                                       state->window_height);
                   }
@@ -1462,6 +1462,7 @@ gwy_data_field_correlation_search(GwyDataField *dfield,
 
     g_object_unref(kernel_weight);
     g_object_unref(kappa);
+    gwy_data_field_invalidate(target);
 }
 
 /************************** Documentation ****************************/
