@@ -1,5 +1,5 @@
 /*
- *  $Id: elliptic.c 21458 2018-09-23 21:43:40Z yeti-dn $
+ *  $Id: elliptic.c 23308 2021-03-18 17:39:48Z yeti-dn $
  *  Copyright (C) 2005-2017 David Necas (Yeti), Petr Klapetek, Chris Anderson.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, sidewinder.asu@gmail.com.
  *
@@ -25,6 +25,76 @@
 #include <libprocess/elliptic.h>
 #include "gwyprocessinternal.h"
 
+typedef void (*EllipticAreaRowFunc)(gdouble *row, gint n, gpointer user_data);
+
+static void
+fill_func(gdouble *row, gint n, gpointer user_data)
+{
+    gdouble v = *(gdouble*)user_data;
+
+    while (n--)
+        *(row++) = v;
+}
+
+static void
+copy_in_func(gdouble *row, gint n, gpointer user_data)
+{
+    gdouble *buf = (gdouble*)user_data;
+
+    gwy_assign(row, buf, n);
+}
+
+static void
+extract_func(gdouble *row, gint n, gpointer user_data)
+{
+    gdouble *buf = (gdouble*)user_data;
+
+    gwy_assign(buf, row, n);
+}
+
+static gint
+elliptic_area_do(GwyDataField *data_field,
+                 gint col, gint row,
+                 gint width, gint height,
+                 EllipticAreaRowFunc func, gpointer user_data)
+{
+    gint i, jfrom, jto, xres, yres, count, ifrom, ito, n;
+    gdouble a, b, s;
+    gdouble *d;
+
+    if (!width || !height)                    /* Compatibility */
+        return 0;
+    g_return_val_if_fail(GWY_IS_DATA_FIELD(data_field), 0);
+
+    a = width/2.0;
+    b = height/2.0;
+    xres = data_field->xres;
+    yres = data_field->yres;
+    count = 0;
+
+    ifrom = MAX(0, row);
+    ito = MIN(row + height-1, yres-1);
+    for (i = ifrom; i <= ito; i++) {
+        d = data_field->data + i*xres;
+        s = (i - row + 0.5)/b;
+        s = s*(2.0 - s);
+        if (s <= 0.0)
+            continue;
+        s = sqrt(s);
+        jfrom = (gint)ceil(a*(1.0 - s) - 0.5) + col;
+        jto = (gint)floor(a*(1.0 + s) - 0.5) + col;
+        jfrom = MAX(jfrom, 0);
+        jto = MIN(jto, xres-1);
+        n = jto - jfrom + 1;
+        g_return_val_if_fail(n >= 0, 0);
+        if (func)
+            func(d + jfrom, n, user_data);
+        count += n;
+    }
+
+    return count;
+}
+
 /**
  * gwy_data_field_elliptic_area_fill:
  * @data_field: A data field.
@@ -36,8 +106,9 @@
  *
  * Fills an elliptic region of a data field with given value.
  *
- * The elliptic region is defined by its bounding box which must be completely
- * contained in the data field.
+ * The elliptic region is defined by its bounding box.  In versions prior to
+ * 2.59 the bounding box must be completely contained in the data field.  Since
+ * version 2.59 the ellipse can intersect the data field in any manner.
  *
  * Returns: The number of filled values.
  **/
@@ -47,41 +118,12 @@ gwy_data_field_elliptic_area_fill(GwyDataField *data_field,
                                   gint width, gint height,
                                   gdouble value)
 {
-    gint i, j, from, to, xres, count;
-    gdouble a, b, s;
-    gdouble *d;
+    gint count;
 
-    if (!width || !height)                    /* Compatibility */
-        return 0;
-    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
-        return 0;
-
-    a = width/2.0;
-    b = height/2.0;
-    xres = data_field->xres;
-    count = 0;
-
-    for (i = 0; i < height; i++) {
-        d = data_field->data + (row + i)*xres + col;
-        s = (i + 0.5)/b;
-        s = s*(2.0 - s);
-        if (G_UNLIKELY(s <= 0.0))
-            continue;
-        s = sqrt(s);
-        from = ceil(a*(1.0 - s) - 0.5);
-        to = floor(a*(1.0 + s) - 0.5);
-        if (G_UNLIKELY(from < 0))
-            from = 0;
-        if (G_UNLIKELY(to >= width))
-            to = width-1;
-        if (G_LIKELY(to >= from)) {
-            for (j = from; j <= to; j++)
-                d[j] = value;
-            count += to - from + 1;
-        }
-    }
-
-    gwy_data_field_invalidate(data_field);
+    count = elliptic_area_do(data_field, col, row, width, height,
+                             fill_func, &value);
+    if (count)
+        gwy_data_field_invalidate(data_field);
 
     return count;
 }
@@ -100,8 +142,9 @@ gwy_data_field_elliptic_area_fill(GwyDataField *data_field,
  *
  * Extracts values from an elliptic region of a data field.
  *
- * The elliptic region is defined by its bounding box which must be completely
- * contained in the data field.
+ * The elliptic region is defined by its bounding box.  In versions prior to
+ * 2.59 the bounding box must be completely contained in the data field.  Since
+ * version 2.59 the ellipse can intersect the data field in any manner.
  *
  * Returns: The number of extracted values.
  **/
@@ -111,40 +154,8 @@ gwy_data_field_elliptic_area_extract(GwyDataField *data_field,
                                      gint width, gint height,
                                      gdouble *data)
 {
-    gint i, from, to, xres, count;
-    gdouble a, b, s;
-    const gdouble *d;
-
-    if (!width || !height)                    /* Compatibility */
-        return 0;
-    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
-        return 0;
-
-    a = width/2.0;
-    b = height/2.0;
-    xres = data_field->xres;
-    count = 0;
-
-    for (i = 0; i < height; i++) {
-        d = data_field->data + (row + i)*xres + col;
-        s = (i + 0.5)/b;
-        s = s*(2.0 - s);
-        if (G_UNLIKELY(s <= 0.0))
-            continue;
-        s = sqrt(s);
-        from = ceil(a*(1.0 - s) - 0.5);
-        to = floor(a*(1.0 + s) - 0.5);
-        if (G_UNLIKELY(from < 0))
-            from = 0;
-        if (G_UNLIKELY(to >= width))
-            to = width-1;
-        if (G_LIKELY(to >= from)) {
-            gwy_assign(data + count, d + from, to - from + 1);
-            count += to - from + 1;
-        }
-    }
-
-    return count;
+    return elliptic_area_do(data_field, col, row, width, height,
+                            extract_func, data);
 }
 
 /**
@@ -159,8 +170,9 @@ gwy_data_field_elliptic_area_extract(GwyDataField *data_field,
  *
  * Puts values back to an elliptic region of a data field.
  *
- * The elliptic region is defined by its bounding box which must be completely
- * contained in the data field.
+ * The elliptic region is defined by its bounding box.  In versions prior to
+ * 2.59 the bounding box must be completely contained in the data field.  Since
+ * version 2.59 the ellipse can intersect the data field in any manner.
  *
  * This method does the reverse of gwy_data_field_elliptic_area_extract()
  * allowing to implement pixel-wise filters on elliptic areas.  Values from
@@ -173,40 +185,33 @@ gwy_data_field_elliptic_area_unextract(GwyDataField *data_field,
                                        gint width, gint height,
                                        const gdouble *data)
 {
-    gint i, from, to, xres, count;
-    gdouble a, b, s;
-    gdouble *d;
+    gint count;
 
-    if (!width || !height)                    /* Compatibility */
-        return;
-    if (!_gwy_data_field_check_area(data_field, col, row, width, height))
-        return;
+    count = elliptic_area_do(data_field, col, row, width, height,
+                             copy_in_func, (gpointer)data);
+    if (count)
+        gwy_data_field_invalidate(data_field);
+}
 
-    a = width/2.0;
-    b = height/2.0;
-    xres = data_field->xres;
-    count = 0;
-
-    for (i = 0; i < height; i++) {
-        d = data_field->data + (row + i)*xres + col;
-        s = (i + 0.5)/b;
-        s = s*(2.0 - s);
-        if (G_UNLIKELY(s <= 0.0))
-            continue;
-        s = sqrt(s);
-        from = ceil(a*(1.0 - s) - 0.5);
-        to = floor(a*(1.0 + s) - 0.5);
-        if (G_UNLIKELY(from < 0))
-            from = 0;
-        if (G_UNLIKELY(to >= width))
-            to = width-1;
-        if (G_LIKELY(to >= from)) {
-            gwy_assign(d + from, data + count, to - from + 1);
-            count += to - from + 1;
-        }
-    }
-
-    gwy_data_field_invalidate(data_field);
+/**
+ * gwy_data_field_get_elliptic_intersection:
+ * @width: Bounding box width.
+ * @height: Bounding box height.
+ *
+ * Calculates an upper bound of the number of samples in an elliptic region
+ * intersecting a data field.
+ *
+ * Returns: The number of pixels in an elliptic region with given rectangular
+ *          bounds (or its upper bound).
+ *
+ * Since: 2.59
+ **/
+gint
+gwy_data_field_get_elliptic_intersection(GwyDataField *data_field,
+                                         gint col, gint row,
+                                         gint width, gint height)
+{
+    return elliptic_area_do(data_field, col, row, width, height, NULL, NULL);
 }
 
 /**
@@ -216,6 +221,11 @@ gwy_data_field_elliptic_area_unextract(GwyDataField *data_field,
  *
  * Calculates an upper bound of the number of samples in an elliptic region.
  *
+ * This function is useful for elliptic areas more or less contained within
+ * the data field.  Otherwise the returned size can be overestimated a lot. Use
+ * gwy_data_field_get_elliptic_intersection() for elliptic areas intersecting
+ * the data field in arbitrary manner.
+ *
  * Returns: The number of pixels in an elliptic region with given rectangular
  *          bounds (or its upper bound).
  **/
@@ -236,7 +246,7 @@ gwy_data_field_get_elliptic_area_size(gint width,
     for (i = 0; i < height; i++) {
         s = (i + 0.5)/b;
         s = s*(2.0 - s);
-        if (G_UNLIKELY(s <= 0))
+        if (s <= 0)
             continue;
         s = sqrt(s);
         from = ceil(a*(1.0 - s) - 0.5);
