@@ -53,6 +53,7 @@ GwyModuleInfo* _gwy_module_query__intelliwave(void);
 GwyModuleInfo* _gwy_module_query__intematix(void);
 GwyModuleInfo* _gwy_module_query__iso28600(void);
 GwyModuleInfo* _gwy_module_query__jeol(void);
+GwyModuleInfo* _gwy_module_query__jeoltem(void);
 GwyModuleInfo* _gwy_module_query__jspmfile(void);
 GwyModuleInfo* _gwy_module_query__keyence(void);
 GwyModuleInfo* _gwy_module_query__leica(void);
@@ -91,6 +92,7 @@ GwyModuleInfo* _gwy_module_query__pltfile(void);
 GwyModuleInfo* _gwy_module_query__princetonspe(void);
 GwyModuleInfo* _gwy_module_query__psia(void);
 GwyModuleInfo* _gwy_module_query__pt3file(void);
+GwyModuleInfo* _gwy_module_query__quazarnpic(void);
 GwyModuleInfo* _gwy_module_query__quesant(void);
 GwyModuleInfo* _gwy_module_query__rawfile(void);
 GwyModuleInfo* _gwy_module_query__rawgraph(void);
@@ -170,6 +172,7 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__intematix, "intematix", },
   { _gwy_module_query__iso28600, "iso28600", },
   { _gwy_module_query__jeol, "jeol", },
+  { _gwy_module_query__jeoltem, "jeoltem", },
   { _gwy_module_query__jspmfile, "jspmfile", },
   { _gwy_module_query__keyence, "keyence", },
   { _gwy_module_query__leica, "leica", },
@@ -208,6 +211,7 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__princetonspe, "princetonspe", },
   { _gwy_module_query__psia, "psia", },
   { _gwy_module_query__pt3file, "pt3file", },
+  { _gwy_module_query__quazarnpic, "quazarnpic", },
   { _gwy_module_query__quesant, "quesant", },
   { _gwy_module_query__rawfile, "rawfile", },
   { _gwy_module_query__rawgraph, "rawgraph", },
