@@ -1,5 +1,5 @@
 /*
- *  $Id: leica.c 20676 2017-12-18 18:19:09Z yeti-dn $
+ *  $Id: leica.c 22562 2019-10-10 14:45:41Z yeti-dn $
  *  Copyright (C) 2014 Daniil Bratashov (dn2010), David Necas (Yeti)..
  *
  *  E-mail: dn2010@gmail.com.
@@ -175,7 +175,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Leica CLSM image files (LIF)."),
     "Daniil Bratashov <dn2010@gmail.com>",
-    "0.2",
+    "0.4",
     "Daniil Bratashov (dn2010), David Necas (Yeti)",
     "2016",
 };
@@ -227,7 +227,7 @@ lif_load(const gchar *filename,
     gsize size = 0, memblock_size = 0;
     gint64 remaining = 0;
     gchar *buffer;
-    const guchar *p;
+    const guchar *p, *tp;
     GError *err = NULL;
     GwyDataField *dfield = NULL;
     GwyBrick *brick = NULL;
@@ -248,6 +248,7 @@ lif_load(const gchar *filename,
     GwySIUnit *siunitx = NULL, *siunity = NULL, *siunitw = NULL;
     gint power10xy = 1;
     gint power10x = 1, power10y = 1, power10z = 1, power10w = 1;
+    gboolean flipz = FALSE;
 
     if (!g_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -287,7 +288,9 @@ lif_load(const gchar *filename,
 
     remaining -= header->xmllen * 2;
 
+    // Uncomment to see large XML header
     // gwy_debug("%s", header->xmlheader);
+
     /* Parse XML header */
     xmldata = g_new0(XMLParserData, 1);
     xmldata->file = g_new0(LIFFile, 1);
@@ -432,11 +435,18 @@ lif_load(const gchar *filename,
                                            yoffset*pow10(power10xy));
 
                 data = gwy_data_field_get_data(dfield);
-                for (y = 0; y < yres; y++)
-                    for (x = 0; x < xres; x++) {
-                        *(data++) = zscale
-                           * (gdouble)*(p + offset + x*xstep + y*ystep);
-                }
+                if (xstep == 1)
+                    for (y = 0; y < yres; y++)
+                        for (x = 0; x < xres; x++) {
+                            *(data++) = zscale * (gdouble)*(p + offset
+                                                   + x*xstep + y*ystep);
+                        }
+                else if (xstep == 2)
+                    for (y = 0; y < yres; y++)
+                        for (x = 0; x < xres; x++) {
+                            tp = p + offset + x*xstep + y*ystep;
+                            *(data++) = zscale * gwy_get_guint16_le(&tp);
+                        }
 
                 if (siunitxy) {
                     gwy_data_field_set_si_unit_xy(dfield, siunitxy);
@@ -505,6 +515,7 @@ lif_load(const gchar *filename,
                                            LIFDimension, 0);
                 xres = dimension->res;
                 xreal = dimension->length;
+                gwy_debug("xreal = %g", xreal);
                 xoffset = dimension->origin;
                 xstep = dimension->bytesinc;
                 siunitx = gwy_si_unit_new_parse(dimension->unit,
@@ -514,6 +525,7 @@ lif_load(const gchar *filename,
                                            LIFDimension, 1);
                 yres = dimension->res;
                 yreal = dimension->length;
+                gwy_debug("yreal = %g", yreal);
                 yoffset = dimension->origin;
                 ystep = dimension->bytesinc;
                 siunity = gwy_si_unit_new_parse(dimension->unit,
@@ -529,7 +541,13 @@ lif_load(const gchar *filename,
                 }
                 zres = dimension->res;
                 zreal = dimension->length;
+                gwy_debug("zreal = %g", zreal);
                 zoffset = dimension->origin;
+                if (zreal < 0.0) {
+                    zreal = -zreal;
+                    zoffset -= zreal;
+                    flipz = TRUE;
+                }
                 zstep = dimension->bytesinc;
                 siunitz = gwy_si_unit_new_parse(dimension->unit,
                                                 &power10z);
@@ -567,12 +585,44 @@ lif_load(const gchar *filename,
 
                 data = gwy_brick_get_data(brick);
 
-                for (z = 0; z < zres; z++)
-                    for (y = 0; y < yres; y++)
-                        for (x = 0; x < xres; x++) {
-                            *(data++) = wscale * (gdouble)*(p + offset
+                if (!flipz) {
+                    if (xstep == 1)
+                        for (z = 0; z < zres; z++)
+                            for (y = 0; y < yres; y++)
+                                for (x = 0; x < xres; x++) {
+                                    *(data++) = wscale * (gdouble)*(p
+                                         + offset
                                          + x*xstep + y*ystep + z*zstep);
-                    }
+                                }
+                    else if (xstep == 2)
+                        for (z = 0; z < zres; z++)
+                            for (y = 0; y < yres; y++)
+                                for (x = 0; x < xres; x++) {
+                                    tp = p + offset
+                                          + x*xstep + y*ystep + z*zstep;
+                                    *(data++)
+                                     = wscale * gwy_get_guint16_le(&tp);
+                                }
+                }
+                else {
+                    if (xstep == 1)
+                        for (z = zres-1; z >= 0; z--)
+                            for (y = 0; y < yres; y++)
+                                for (x = 0; x < xres; x++) {
+                                    *(data++) = wscale * (gdouble)*(p
+                                         + offset
+                                         + x*xstep + y*ystep + z*zstep);
+                                }
+                    else if (xstep == 2)
+                        for (z = zres-1; z >= 0; z--)
+                            for (y = 0; y < yres; y++)
+                                for (x = 0; x < xres; x++) {
+                                    tp = p + offset
+                                          + x*xstep + y*ystep + z*zstep;
+                                    *(data++)
+                                     = wscale * gwy_get_guint16_le(&tp);
+                                }
+                }
 
                 if (siunitx) {
                     gwy_brick_set_si_unit_x(brick, siunitx);
@@ -634,19 +684,7 @@ lif_load(const gchar *filename,
                     }
                 }
 
-                dfield = gwy_data_field_new(xres, yres,
-                                            xreal, yreal, FALSE);
-                gwy_brick_mean_plane(brick, dfield,
-                                     0, 0, 0,
-                                     xres, yres, -1, FALSE);
-                strkey = g_strdup_printf("/brick/%d/preview",
-                                         volumeno);
-                gwy_container_set_object_by_name(container,
-                                                 strkey,
-                                                 dfield);
-                g_free(strkey);
                 g_object_unref(brick);
-                g_object_unref(dfield);
                 gwy_file_volume_import_log_add(container, volumeno,
                                                NULL, filename);
 
