@@ -1,5 +1,5 @@
 /*
- *  $Id: psia.c 22642 2019-11-03 11:46:07Z yeti-dn $
+ *  $Id: psia.c 23870 2021-06-23 11:53:07Z yeti-dn $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -245,7 +245,7 @@ static GwyModuleInfo module_info = {
     module_register,
     N_("Imports Park Systems data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.7",
+    "0.8",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -321,15 +321,13 @@ psia_load_tiff(GwyTIFF *tiff, GError **error)
     PSIAImageHeader header;
     GwyContainer *container = NULL;
     GwyContainer *meta = NULL;
-    GwyDataField *dfield, *ccfield;
+    GwyDataField *dfield;
     GwySIUnit *siunit;
-    guint magic, version, bps, i, j;
+    guint magic, version, bps, i;
     const guchar *p, *data;
     gchar *comment = NULL;
     gint count, data_len, power10;
     gdouble q, z0;
-    gdouble *dd;
-    const gdouble *sd;
 
     if (!gwy_tiff_get_uint0(tiff, PSIA_TIFFTAG_MagicNumber, &magic)
         || magic != PSIA_MAGIC_NUMBER
@@ -409,36 +407,9 @@ psia_load_tiff(GwyTIFF *tiff, GError **error)
     container = gwy_container_new();
 
     gwy_data_field_invert(dfield, TRUE, FALSE, FALSE);
-    if (header.swap_xy) {
-        /*
-         *  rotating datafield CCW to display and process horizontally
-         *  without losing values that get outside of the datafield
-         */
-        ccfield = gwy_data_field_new_alike(dfield, FALSE);
-        gwy_data_field_set_xreal(ccfield, header.yreal);
-        gwy_data_field_set_yreal(ccfield, header.xreal);
-
-        gwy_data_field_resample(ccfield,
-                                header.yres,
-                                header.xres,
-                                GWY_INTERPOLATION_NONE);
-
-        sd = gwy_data_field_get_data_const(dfield);
-        dd = gwy_data_field_get_data(ccfield);
-
-        for (i = 0; i < header.xres; i++) {
-            for (j = 0; j < header.yres; j++) {
-                dd[i*header.yres + j] = sd[j*header.xres + (header.xres - 1 - i)];
-            }
-        }
-
-        gwy_container_set_object_by_name(container, "/0/data", ccfield);
-        g_object_unref(ccfield);
-    }
-    else {
-        gwy_container_set_object_by_name(container, "/0/data", dfield);
-    }
+    gwy_container_set_object_by_name(container, "/0/data", dfield);
     g_object_unref(dfield);
+    gwy_app_channel_check_nonsquare(container, 0);
 
     if (header.source_name && *header.source_name)
         gwy_container_set_string_by_name(container, "/0/data/title",
