@@ -1,5 +1,5 @@
 /*
- *  $Id: calibrate.c 22501 2019-09-19 14:51:31Z yeti-dn $
+ *  $Id: calibrate.c 23681 2021-05-12 22:59:50Z yeti-dn $
  *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -533,7 +533,7 @@ calibrate_dialog(CalibrateArgs *args,
     gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 4);
     row++;
 
-    controls.square = gtk_check_button_new_with_mnemonic(_("_Square samples"));
+    controls.square = gtk_check_button_new_with_mnemonic(_("_Square pixels"));
     gtk_table_attach(GTK_TABLE(table), controls.square,
                      0, 2, row, row+1, GTK_FILL, 0, 0, 0);
     row++;
@@ -744,10 +744,8 @@ dialog_reset(CalibrateControls *controls,
     args->x0 = args->x0orig;
     args->y0 = args->y0orig;
 
-    g_free(args->zunit);
-    args->zunit = g_strdup(args->zunitorig);
-    g_free(args->xyunit);
-    args->xyunit = g_strdup(args->xyunitorig);
+    gwy_assign_string(&args->zunit, args->zunitorig);gwy_assign_string(&args->xyunit,
+                                                                       args->xyunitorig);
 }
 
 static void
@@ -1114,13 +1112,11 @@ units_change(GtkWidget *button,
     unit = gtk_entry_get_text(GTK_ENTRY(entry));
     if (gwy_strequal(id, "xy")) {
         set_combo_from_unit(controls->xyexponent, unit, 0);
-        g_free(controls->args->xyunit);
-        controls->args->xyunit = g_strdup(unit);
+        gwy_assign_string(&controls->args->xyunit, unit);
     }
     else if (gwy_strequal(id, "z")) {
         set_combo_from_unit(controls->zexponent, unit, 0);
-        g_free(controls->args->zunit);
-        controls->args->zunit = g_strdup(unit);
+        gwy_assign_string(&controls->args->zunit, unit);
     }
 
     gtk_widget_destroy(dialog);
