@@ -1,5 +1,5 @@
 /*
- *  $Id: deposit_synth.c 22405 2019-08-18 20:02:55Z yeti-dn $
+ *  $Id: deposit_synth.c 23556 2021-04-21 08:00:51Z yeti-dn $
  *  Copyright (C) 2007,2009,2010,2017 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -368,7 +368,7 @@ deposit_synth_dialog(DepositSynthArgs *args,
     }
 
     gwy_container_set_object_by_name(controls.mydata, "/0/data", dfield);
-    controls.view = create_preview(controls.mydata, 0, PREVIEW_SIZE, FALSE);
+    controls.view = gwy_create_preview(controls.mydata, 0, PREVIEW_SIZE, FALSE);
     gtk_box_pack_start(GTK_BOX(vbox), controls.view, FALSE, FALSE, 0);
 
     gtk_box_pack_start(GTK_BOX(vbox),
