@@ -1,5 +1,5 @@
 /*
- *  $Id: facet_analysis.c 22875 2020-07-01 08:56:37Z yeti-dn $
+ *  $Id: facet_analysis.c 23559 2021-04-23 23:08:22Z yeti-dn $
  *  Copyright (C) 2003-2017 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -355,22 +355,8 @@ facets_analyse(GwyContainer *data, GwyRunType run)
                                      0);
     g_return_if_fail(dfield && mquark);
 
-    if (!gwy_si_unit_equal(gwy_data_field_get_si_unit_xy(dfield),
-                           gwy_data_field_get_si_unit_z(dfield))) {
-        GtkWidget *dialog;
-
-        dialog = gtk_message_dialog_new
-                        (gwy_app_find_window_for_channel(data, id),
-                         GTK_DIALOG_DESTROY_WITH_PARENT,
-                         GTK_MESSAGE_ERROR,
-                         GTK_BUTTONS_OK,
-                         _("%s: Lateral dimensions and value must "
-                           "be the same physical quantity."),
-                         _("Facet Analysis"));
-        gtk_dialog_run(GTK_DIALOG(dialog));
-        gtk_widget_destroy(dialog);
+    if (!gwy_require_image_same_units(dfield, data, id, _("Facet Analysis")))
         return;
-    }
 
     facets_load_args(gwy_app_settings_get(), &args);
     if (run == GWY_RUN_IMMEDIATE) {
@@ -442,7 +428,7 @@ facets_dialog(FacetsArgs *args,
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox,
                        FALSE, FALSE, 2);
 
-    controls.view = create_preview(controls.mydata, 0, IMAGEVIEW_SIZE,
+    controls.view = gwy_create_preview(controls.mydata, 0, IMAGEVIEW_SIZE,
                                    TRUE);
     gtk_box_pack_start(GTK_BOX(hbox), controls.view, FALSE, FALSE, 4);
     selection = create_vector_layer(GWY_DATA_VIEW(controls.view), 0, "Point",
@@ -529,13 +515,16 @@ facets_dialog(FacetsArgs *args,
     vbox = gtk_vbox_new(FALSE, 2);
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 4);
 
-    controls.fview = create_preview(controls.fdata, 0, FACETVIEW_SIZE, TRUE);
+    controls.fview = gwy_create_preview(controls.fdata, 0, FACETVIEW_SIZE, TRUE);
     gtk_box_pack_start(GTK_BOX(vbox), controls.fview, FALSE, FALSE, 0);
     selection = create_vector_layer(GWY_DATA_VIEW(controls.fview), 0, "Point",
                                     TRUE);
     controls.fselection = selection;
     gwy_selection_set_max_objects(selection, 1024);
     selkey = g_strdup_printf("/%d/select/_facets", id);
+    /* XXX XXX XXX: This is bogus. The selection depends on q, which changes
+     * with plane size. We have to remember the selection as derivatives, not
+     * facet view coordinates – in sync with facet_measure. */
     if (gwy_container_gis_object_by_name(data, selkey, &selection)) {
         gwy_selection_assign(controls.fselection, selection);
         selection = controls.fselection;
@@ -1428,7 +1417,8 @@ run_noninteractive(FacetsArgs *args,
     GwyDataField *mask;
 
     gwy_app_undo_qcheckpointv(data, 1, &mquark);
-    mask = create_mask_field(dfield);
+    mask = gwy_data_field_new_alike(dfield, TRUE);
+    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mask), NULL);
 
     gwy_data_field_mark_facets(dtheta, dphi, theta, phi, args->tolerance, mask);
     if (mfield && args->combine) {
@@ -1924,7 +1914,8 @@ add_mask_field(GwyDataView *view,
         return;
 
     gwy_container_gis_object_by_name(data, "/0/data", &dfield);
-    mfield = create_mask_field(dfield);
+    mfield = gwy_data_field_new_alike(dfield, TRUE);
+    gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mfield), NULL);
     gwy_container_set_object_by_name(data, "/0/mask", mfield);
     g_object_unref(mfield);
     if (color)
