@@ -1,5 +1,5 @@
 /*
- *  $Id: otsu_threshold.c 21376 2018-08-30 14:34:29Z yeti-dn $
+ *  $Id: otsu_threshold.c 23557 2021-04-22 07:34:37Z yeti-dn $
  *  Copyright (C) 2013 Brazilian Nanotechnology National Laboratory
  *  E-mail: Vinicius Barboza <vinicius.barboza@lnnano.cnpem.br>
  *
@@ -94,7 +94,8 @@ otsu_threshold(GwyContainer *data,
 
     /* Checking for mask and creating a new one */
     if (!mfield) {
-        mfield = create_mask_field(dfield);
+        mfield = gwy_data_field_new_alike(dfield, TRUE);
+        gwy_si_unit_set_from_string(gwy_data_field_get_si_unit_z(mfield), NULL);
         gwy_container_set_object(data, mquark, mfield);
         g_object_unref(mfield);
     }
