@@ -15,6 +15,7 @@ GWY_MODULE_QUERY(module_info)
 GwyModuleInfo* _gwy_module_query__acf2d(void);
 GwyModuleInfo* _gwy_module_query__angle_dist(void);
 GwyModuleInfo* _gwy_module_query__anneal_synth(void);
+GwyModuleInfo* _gwy_module_query__arc_revolve(void);
 GwyModuleInfo* _gwy_module_query__arithmetic(void);
 GwyModuleInfo* _gwy_module_query__averaging(void);
 GwyModuleInfo* _gwy_module_query__basicops(void);
@@ -31,6 +32,7 @@ GwyModuleInfo* _gwy_module_query__col_synth(void);
 GwyModuleInfo* _gwy_module_query__convolution_filter(void);
 GwyModuleInfo* _gwy_module_query__convolve(void);
 GwyModuleInfo* _gwy_module_query__correct_affine(void);
+GwyModuleInfo* _gwy_module_query__correct_perspective(void);
 GwyModuleInfo* _gwy_module_query__cpde_synth(void);
 GwyModuleInfo* _gwy_module_query__crosscor(void);
 GwyModuleInfo* _gwy_module_query__curvature(void);
@@ -70,8 +72,10 @@ GwyModuleInfo* _gwy_module_query__grain_cross(void);
 GwyModuleInfo* _gwy_module_query__grain_dist(void);
 GwyModuleInfo* _gwy_module_query__grain_edge(void);
 GwyModuleInfo* _gwy_module_query__grain_filter(void);
+GwyModuleInfo* _gwy_module_query__grain_makesel(void);
 GwyModuleInfo* _gwy_module_query__grain_mark(void);
 GwyModuleInfo* _gwy_module_query__grain_stat(void);
+GwyModuleInfo* _gwy_module_query__grain_summary(void);
 GwyModuleInfo* _gwy_module_query__grain_wshed(void);
 GwyModuleInfo* _gwy_module_query__hertz(void);
 GwyModuleInfo* _gwy_module_query__hough(void);
@@ -145,7 +149,6 @@ GwyModuleInfo* _gwy_module_query__tip_model(void);
 GwyModuleInfo* _gwy_module_query__tipops(void);
 GwyModuleInfo* _gwy_module_query__tipshape(void);
 GwyModuleInfo* _gwy_module_query__trimmed_mean(void);
-GwyModuleInfo* _gwy_module_query__unrotate(void);
 GwyModuleInfo* _gwy_module_query__volumize(void);
 GwyModuleInfo* _gwy_module_query__volumize_layers(void);
 GwyModuleInfo* _gwy_module_query__wave_synth(void);
@@ -160,6 +163,7 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__acf2d, "acf2d", },
   { _gwy_module_query__angle_dist, "angle_dist", },
   { _gwy_module_query__anneal_synth, "anneal_synth", },
+  { _gwy_module_query__arc_revolve, "arc-revolve", },
   { _gwy_module_query__arithmetic, "arithmetic", },
   { _gwy_module_query__averaging, "averaging", },
   { _gwy_module_query__basicops, "basicops", },
@@ -176,6 +180,7 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__convolution_filter, "convolution_filter", },
   { _gwy_module_query__convolve, "convolve", },
   { _gwy_module_query__correct_affine, "correct_affine", },
+  { _gwy_module_query__correct_perspective, "correct_perspective", },
   { _gwy_module_query__cpde_synth, "cpde_synth", },
   { _gwy_module_query__crosscor, "crosscor", },
   { _gwy_module_query__curvature, "curvature", },
@@ -215,8 +220,10 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__grain_dist, "grain_dist", },
   { _gwy_module_query__grain_edge, "grain_edge", },
   { _gwy_module_query__grain_filter, "grain_filter", },
+  { _gwy_module_query__grain_makesel, "grain_makesel", },
   { _gwy_module_query__grain_mark, "grain_mark", },
   { _gwy_module_query__grain_stat, "grain_stat", },
+  { _gwy_module_query__grain_summary, "grain_summary", },
   { _gwy_module_query__grain_wshed, "grain_wshed", },
   { _gwy_module_query__hertz, "hertz", },
   { _gwy_module_query__hough, "hough", },
@@ -290,7 +297,6 @@ static const GwyModuleRecord modules[] = {
   { _gwy_module_query__tipops, "tipops", },
   { _gwy_module_query__tipshape, "tipshape", },
   { _gwy_module_query__trimmed_mean, "trimmed-mean", },
-  { _gwy_module_query__unrotate, "unrotate", },
   { _gwy_module_query__volumize, "volumize", },
   { _gwy_module_query__volumize_layers, "volumize_layers", },
   { _gwy_module_query__wave_synth, "wave_synth", },
