@@ -1,5 +1,5 @@
 /*
- *  $Id: linestats.c 22769 2020-02-19 16:49:52Z yeti-dn $
+ *  $Id: linestats.c 23305 2021-03-18 14:40:27Z yeti-dn $
  *  Copyright (C) 2003-2018 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -423,7 +423,7 @@ gwy_tool_line_stats_init_dialog(GwyToolLineStats *tool)
                                         GTK_RESPONSE_APPLY);
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_APPLY, FALSE);
-    gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_NO_BUTTON);
+    gwy_help_add_to_tool_dialog(dialog, GWY_TOOL(tool), GWY_HELP_DEFAULT);
 
     update_sensitivity(tool);
 
